# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import proto  # type: ignore

from google.cloud.aiplatform_v1.types import (
    feature_online_store as gca_feature_online_store,
)
from google.cloud.aiplatform_v1.types import feature_view as gca_feature_view
from google.cloud.aiplatform_v1.types import feature_view_sync as gca_feature_view_sync
from google.cloud.aiplatform_v1.types import operation
from google.protobuf import field_mask_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.cloud.aiplatform.v1",
    manifest={
        "CreateFeatureOnlineStoreRequest",
        "GetFeatureOnlineStoreRequest",
        "ListFeatureOnlineStoresRequest",
        "ListFeatureOnlineStoresResponse",
        "UpdateFeatureOnlineStoreRequest",
        "DeleteFeatureOnlineStoreRequest",
        "CreateFeatureViewRequest",
        "GetFeatureViewRequest",
        "ListFeatureViewsRequest",
        "ListFeatureViewsResponse",
        "UpdateFeatureViewRequest",
        "DeleteFeatureViewRequest",
        "CreateFeatureOnlineStoreOperationMetadata",
        "UpdateFeatureOnlineStoreOperationMetadata",
        "CreateFeatureViewOperationMetadata",
        "UpdateFeatureViewOperationMetadata",
        "SyncFeatureViewRequest",
        "SyncFeatureViewResponse",
        "GetFeatureViewSyncRequest",
        "ListFeatureViewSyncsRequest",
        "ListFeatureViewSyncsResponse",
    },
)


class CreateFeatureOnlineStoreRequest(proto.Message):
    r"""Request message for
    [FeatureOnlineStoreAdminService.CreateFeatureOnlineStore][google.cloud.aiplatform.v1.FeatureOnlineStoreAdminService.CreateFeatureOnlineStore].

    Attributes:
        parent (str):
            Required. The resource name of the Location to create
            FeatureOnlineStores. Format:
            ``projects/{project}/locations/{location}``
        feature_online_store (google.cloud.aiplatform_v1.types.FeatureOnlineStore):
            Required. The FeatureOnlineStore to create.
        feature_online_store_id (str):
            Required. The ID to use for this FeatureOnlineStore, which
            will become the final component of the FeatureOnlineStore's
            resource name.

            This value may be up to 60 characters, and valid characters
            are ``[a-z0-9_]``. The first character cannot be a number.

            The value must be unique within the project and location.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    feature_online_store: gca_feature_online_store.FeatureOnlineStore = proto.Field(
        proto.MESSAGE,
        number=2,
        message=gca_feature_online_store.FeatureOnlineStore,
    )
    feature_online_store_id: str = proto.Field(
        proto.STRING,
        number=3,
    )


class GetFeatureOnlineStoreRequest(proto.Message):
    r"""Request message for
    [FeatureOnlineStoreAdminService.GetFeatureOnlineStore][google.cloud.aiplatform.v1.FeatureOnlineStoreAdminService.GetFeatureOnlineStore].

    Attributes:
        name (str):
            Required. The name of the FeatureOnlineStore
            resource.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ListFeatureOnlineStoresRequest(proto.Message):
    r"""Request message for
    [FeatureOnlineStoreAdminService.ListFeatureOnlineStores][google.cloud.aiplatform.v1.FeatureOnlineStoreAdminService.ListFeatureOnlineStores].

    Attributes:
        parent (str):
            Required. The resource name of the Location to list
            FeatureOnlineStores. Format:
            ``projects/{project}/locations/{location}``
        filter (str):
            Lists the FeatureOnlineStores that match the filter
            expression. The following fields are supported:

            - ``create_time``: Supports ``=``, ``!=``, ``<``, ``>``,
              ``<=``, and ``>=`` comparisons. Values must be in RFC 3339
              format.
            - ``update_time``: Supports ``=``, ``!=``, ``<``, ``>``,
              ``<=``, and ``>=`` comparisons. Values must be in RFC 3339
              format.
            - ``labels``: Supports key-value equality and key presence.

            Examples:

            - ``create_time > "2020-01-01" OR update_time > "2020-01-01"``
              FeatureOnlineStores created or updated after 2020-01-01.
            - ``labels.env = "prod"`` FeatureOnlineStores with label
              "env" set to "prod".
        page_size (int):
            The maximum number of FeatureOnlineStores to
            return. The service may return fewer than this
            value. If unspecified, at most 100
            FeatureOnlineStores will be returned. The
            maximum value is 100; any value greater than 100
            will be coerced to 100.
        page_token (str):
            A page token, received from a previous
            [FeatureOnlineStoreAdminService.ListFeatureOnlineStores][google.cloud.aiplatform.v1.FeatureOnlineStoreAdminService.ListFeatureOnlineStores]
            call. Provide this to retrieve the subsequent page.

            When paginating, all other parameters provided to
            [FeatureOnlineStoreAdminService.ListFeatureOnlineStores][google.cloud.aiplatform.v1.FeatureOnlineStoreAdminService.ListFeatureOnlineStores]
            must match the call that provided the page token.
        order_by (str):
            A comma-separated list of fields to order by, sorted in
            ascending order. Use "desc" after a field name for
            descending. Supported Fields:

            - ``create_time``
            - ``update_time``
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    filter: str = proto.Field(
        proto.STRING,
        number=2,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=3,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=4,
    )
    order_by: str = proto.Field(
        proto.STRING,
        number=5,
    )


class ListFeatureOnlineStoresResponse(proto.Message):
    r"""Response message for
    [FeatureOnlineStoreAdminService.ListFeatureOnlineStores][google.cloud.aiplatform.v1.FeatureOnlineStoreAdminService.ListFeatureOnlineStores].

    Attributes:
        feature_online_stores (MutableSequence[google.cloud.aiplatform_v1.types.FeatureOnlineStore]):
            The FeatureOnlineStores matching the request.
        next_page_token (str):
            A token, which can be sent as
            [ListFeatureOnlineStoresRequest.page_token][google.cloud.aiplatform.v1.ListFeatureOnlineStoresRequest.page_token]
            to retrieve the next page. If this field is omitted, there
            are no subsequent pages.
    """

    @property
    def raw_page(self):
        return self

    feature_online_stores: MutableSequence[
        gca_feature_online_store.FeatureOnlineStore
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=gca_feature_online_store.FeatureOnlineStore,
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class UpdateFeatureOnlineStoreRequest(proto.Message):
    r"""Request message for
    [FeatureOnlineStoreAdminService.UpdateFeatureOnlineStore][google.cloud.aiplatform.v1.FeatureOnlineStoreAdminService.UpdateFeatureOnlineStore].

    Attributes:
        feature_online_store (google.cloud.aiplatform_v1.types.FeatureOnlineStore):
            Required. The FeatureOnlineStore's ``name`` field is used to
            identify the FeatureOnlineStore to be updated. Format:
            ``projects/{project}/locations/{location}/featureOnlineStores/{feature_online_store}``
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            Field mask is used to specify the fields to be overwritten
            in the FeatureOnlineStore resource by the update. The fields
            specified in the update_mask are relative to the resource,
            not the full request. A field will be overwritten if it is
            in the mask. If the user does not provide a mask then only
            the non-empty fields present in the request will be
            overwritten. Set the update_mask to ``*`` to override all
            fields.

            Updatable fields:

            - ``labels``
            - ``description``
            - ``bigtable``
            - ``bigtable.auto_scaling``
            - ``bigtable.enable_multi_region_replica``
    """

    feature_online_store: gca_feature_online_store.FeatureOnlineStore = proto.Field(
        proto.MESSAGE,
        number=1,
        message=gca_feature_online_store.FeatureOnlineStore,
    )
    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=2,
        message=field_mask_pb2.FieldMask,
    )


class DeleteFeatureOnlineStoreRequest(proto.Message):
    r"""Request message for
    [FeatureOnlineStoreAdminService.DeleteFeatureOnlineStore][google.cloud.aiplatform.v1.FeatureOnlineStoreAdminService.DeleteFeatureOnlineStore].

    Attributes:
        name (str):
            Required. The name of the FeatureOnlineStore to be deleted.
            Format:
            ``projects/{project}/locations/{location}/featureOnlineStores/{feature_online_store}``
        force (bool):
            If set to true, any FeatureViews and Features
            for this FeatureOnlineStore will also be
            deleted. (Otherwise, the request will only work
            if the FeatureOnlineStore has no FeatureViews.)
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    force: bool = proto.Field(
        proto.BOOL,
        number=2,
    )


class CreateFeatureViewRequest(proto.Message):
    r"""Request message for
    [FeatureOnlineStoreAdminService.CreateFeatureView][google.cloud.aiplatform.v1.FeatureOnlineStoreAdminService.CreateFeatureView].

    Attributes:
        parent (str):
            Required. The resource name of the FeatureOnlineStore to
            create FeatureViews. Format:
            ``projects/{project}/locations/{location}/featureOnlineStores/{feature_online_store}``
        feature_view (google.cloud.aiplatform_v1.types.FeatureView):
            Required. The FeatureView to create.
        feature_view_id (str):
            Required. The ID to use for the FeatureView, which will
            become the final component of the FeatureView's resource
            name.

            This value may be up to 60 characters, and valid characters
            are ``[a-z0-9_]``. The first character cannot be a number.

            The value must be unique within a FeatureOnlineStore.
        run_sync_immediately (bool):
            Immutable. If set to true, one on demand sync will be run
            immediately, regardless whether the
            [FeatureView.sync_config][google.cloud.aiplatform.v1.FeatureView.sync_config]
            is configured or not.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    feature_view: gca_feature_view.FeatureView = proto.Field(
        proto.MESSAGE,
        number=2,
        message=gca_feature_view.FeatureView,
    )
    feature_view_id: str = proto.Field(
        proto.STRING,
        number=3,
    )
    run_sync_immediately: bool = proto.Field(
        proto.BOOL,
        number=4,
    )


class GetFeatureViewRequest(proto.Message):
    r"""Request message for
    [FeatureOnlineStoreAdminService.GetFeatureView][google.cloud.aiplatform.v1.FeatureOnlineStoreAdminService.GetFeatureView].

    Attributes:
        name (str):
            Required. The name of the FeatureView resource. Format:
            ``projects/{project}/locations/{location}/featureOnlineStores/{feature_online_store}/featureViews/{feature_view}``
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ListFeatureViewsRequest(proto.Message):
    r"""Request message for
    [FeatureOnlineStoreAdminService.ListFeatureViews][google.cloud.aiplatform.v1.FeatureOnlineStoreAdminService.ListFeatureViews].

    Attributes:
        parent (str):
            Required. The resource name of the FeatureOnlineStore to
            list FeatureViews. Format:
            ``projects/{project}/locations/{location}/featureOnlineStores/{feature_online_store}``
        filter (str):
            Lists the FeatureViews that match the filter expression. The
            following filters are supported:

            - ``create_time``: Supports ``=``, ``!=``, ``<``, ``>``,
              ``>=``, and ``<=`` comparisons. Values must be in RFC 3339
              format.
            - ``update_time``: Supports ``=``, ``!=``, ``<``, ``>``,
              ``>=``, and ``<=`` comparisons. Values must be in RFC 3339
              format.
            - ``labels``: Supports key-value equality as well as key
              presence.

            Examples:

            - ``create_time > \"2020-01-31T15:30:00.000000Z\" OR update_time > \"2020-01-31T15:30:00.000000Z\"``
              --> FeatureViews created or updated after
              2020-01-31T15:30:00.000000Z.
            - ``labels.active = yes AND labels.env = prod`` -->
              FeatureViews having both (active: yes) and (env: prod)
              labels.
            - ``labels.env: *`` --> Any FeatureView which has a label
              with 'env' as the key.
        page_size (int):
            The maximum number of FeatureViews to return.
            The service may return fewer than this value. If
            unspecified, at most 1000 FeatureViews will be
            returned. The maximum value is 1000; any value
            greater than 1000 will be coerced to 1000.
        page_token (str):
            A page token, received from a previous
            [FeatureOnlineStoreAdminService.ListFeatureViews][google.cloud.aiplatform.v1.FeatureOnlineStoreAdminService.ListFeatureViews]
            call. Provide this to retrieve the subsequent page.

            When paginating, all other parameters provided to
            [FeatureOnlineStoreAdminService.ListFeatureViews][google.cloud.aiplatform.v1.FeatureOnlineStoreAdminService.ListFeatureViews]
            must match the call that provided the page token.
        order_by (str):
            A comma-separated list of fields to order by, sorted in
            ascending order. Use "desc" after a field name for
            descending.

            Supported fields:

            - ``feature_view_id``
            - ``create_time``
            - ``update_time``
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    filter: str = proto.Field(
        proto.STRING,
        number=2,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=3,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=4,
    )
    order_by: str = proto.Field(
        proto.STRING,
        number=5,
    )


class ListFeatureViewsResponse(proto.Message):
    r"""Response message for
    [FeatureOnlineStoreAdminService.ListFeatureViews][google.cloud.aiplatform.v1.FeatureOnlineStoreAdminService.ListFeatureViews].

    Attributes:
        feature_views (MutableSequence[google.cloud.aiplatform_v1.types.FeatureView]):
            The FeatureViews matching the request.
        next_page_token (str):
            A token, which can be sent as
            [ListFeatureViewsRequest.page_token][google.cloud.aiplatform.v1.ListFeatureViewsRequest.page_token]
            to retrieve the next page. If this field is omitted, there
            are no subsequent pages.
    """

    @property
    def raw_page(self):
        return self

    feature_views: MutableSequence[gca_feature_view.FeatureView] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=gca_feature_view.FeatureView,
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class UpdateFeatureViewRequest(proto.Message):
    r"""Request message for
    [FeatureOnlineStoreAdminService.UpdateFeatureView][google.cloud.aiplatform.v1.FeatureOnlineStoreAdminService.UpdateFeatureView].

    Attributes:
        feature_view (google.cloud.aiplatform_v1.types.FeatureView):
            Required. The FeatureView's ``name`` field is used to
            identify the FeatureView to be updated. Format:
            ``projects/{project}/locations/{location}/featureOnlineStores/{feature_online_store}/featureViews/{feature_view}``
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            Field mask is used to specify the fields to be overwritten
            in the FeatureView resource by the update. The fields
            specified in the update_mask are relative to the resource,
            not the full request. A field will be overwritten if it is
            in the mask. If the user does not provide a mask then only
            the non-empty fields present in the request will be
            overwritten. Set the update_mask to ``*`` to override all
            fields.

            Updatable fields:

            - ``labels``
            - ``service_agent_type``
            - ``big_query_source``
            - ``big_query_source.uri``
            - ``big_query_source.entity_id_columns``
            - ``feature_registry_source``
            - ``feature_registry_source.feature_groups``
            - ``sync_config``
            - ``sync_config.cron``
            - ``optimized_config.automatic_resources``
    """

    feature_view: gca_feature_view.FeatureView = proto.Field(
        proto.MESSAGE,
        number=1,
        message=gca_feature_view.FeatureView,
    )
    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=2,
        message=field_mask_pb2.FieldMask,
    )


class DeleteFeatureViewRequest(proto.Message):
    r"""Request message for
    [FeatureOnlineStoreAdminService.DeleteFeatureView][google.cloud.aiplatform.v1.FeatureOnlineStoreAdminService.DeleteFeatureView].

    Attributes:
        name (str):
            Required. The name of the FeatureView to be deleted. Format:
            ``projects/{project}/locations/{location}/featureOnlineStores/{feature_online_store}/featureViews/{feature_view}``
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class CreateFeatureOnlineStoreOperationMetadata(proto.Message):
    r"""Details of operations that perform create FeatureOnlineStore.

    Attributes:
        generic_metadata (google.cloud.aiplatform_v1.types.GenericOperationMetadata):
            Operation metadata for FeatureOnlineStore.
    """

    generic_metadata: operation.GenericOperationMetadata = proto.Field(
        proto.MESSAGE,
        number=1,
        message=operation.GenericOperationMetadata,
    )


class UpdateFeatureOnlineStoreOperationMetadata(proto.Message):
    r"""Details of operations that perform update FeatureOnlineStore.

    Attributes:
        generic_metadata (google.cloud.aiplatform_v1.types.GenericOperationMetadata):
            Operation metadata for FeatureOnlineStore.
    """

    generic_metadata: operation.GenericOperationMetadata = proto.Field(
        proto.MESSAGE,
        number=1,
        message=operation.GenericOperationMetadata,
    )


class CreateFeatureViewOperationMetadata(proto.Message):
    r"""Details of operations that perform create FeatureView.

    Attributes:
        generic_metadata (google.cloud.aiplatform_v1.types.GenericOperationMetadata):
            Operation metadata for FeatureView Create.
    """

    generic_metadata: operation.GenericOperationMetadata = proto.Field(
        proto.MESSAGE,
        number=1,
        message=operation.GenericOperationMetadata,
    )


class UpdateFeatureViewOperationMetadata(proto.Message):
    r"""Details of operations that perform update FeatureView.

    Attributes:
        generic_metadata (google.cloud.aiplatform_v1.types.GenericOperationMetadata):
            Operation metadata for FeatureView Update.
    """

    generic_metadata: operation.GenericOperationMetadata = proto.Field(
        proto.MESSAGE,
        number=1,
        message=operation.GenericOperationMetadata,
    )


class SyncFeatureViewRequest(proto.Message):
    r"""Request message for
    [FeatureOnlineStoreAdminService.SyncFeatureView][google.cloud.aiplatform.v1.FeatureOnlineStoreAdminService.SyncFeatureView].

    Attributes:
        feature_view (str):
            Required. Format:
            ``projects/{project}/locations/{location}/featureOnlineStores/{feature_online_store}/featureViews/{feature_view}``
    """

    feature_view: str = proto.Field(
        proto.STRING,
        number=1,
    )


class SyncFeatureViewResponse(proto.Message):
    r"""Response message for
    [FeatureOnlineStoreAdminService.SyncFeatureView][google.cloud.aiplatform.v1.FeatureOnlineStoreAdminService.SyncFeatureView].

    Attributes:
        feature_view_sync (str):
            Format:
            ``projects/{project}/locations/{location}/featureOnlineStores/{feature_online_store}/featureViews/{feature_view}/featureViewSyncs/{feature_view_sync}``
    """

    feature_view_sync: str = proto.Field(
        proto.STRING,
        number=1,
    )


class GetFeatureViewSyncRequest(proto.Message):
    r"""Request message for
    [FeatureOnlineStoreAdminService.GetFeatureViewSync][google.cloud.aiplatform.v1.FeatureOnlineStoreAdminService.GetFeatureViewSync].

    Attributes:
        name (str):
            Required. The name of the FeatureViewSync resource. Format:
            ``projects/{project}/locations/{location}/featureOnlineStores/{feature_online_store}/featureViews/{feature_view}/featureViewSyncs/{feature_view_sync}``
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ListFeatureViewSyncsRequest(proto.Message):
    r"""Request message for
    [FeatureOnlineStoreAdminService.ListFeatureViewSyncs][google.cloud.aiplatform.v1.FeatureOnlineStoreAdminService.ListFeatureViewSyncs].

    Attributes:
        parent (str):
            Required. The resource name of the FeatureView to list
            FeatureViewSyncs. Format:
            ``projects/{project}/locations/{location}/featureOnlineStores/{feature_online_store}/featureViews/{feature_view}``
        filter (str):
            Lists the FeatureViewSyncs that match the filter expression.
            The following filters are supported:

            - ``create_time``: Supports ``=``, ``!=``, ``<``, ``>``,
              ``>=``, and ``<=`` comparisons. Values must be in RFC 3339
              format.

            Examples:

            - ``create_time > \"2020-01-31T15:30:00.000000Z\"`` -->
              FeatureViewSyncs created after
              2020-01-31T15:30:00.000000Z.
        page_size (int):
            The maximum number of FeatureViewSyncs to
            return. The service may return fewer than this
            value. If unspecified, at most 1000
            FeatureViewSyncs will be returned. The maximum
            value is 1000; any value greater than 1000 will
            be coerced to 1000.
        page_token (str):
            A page token, received from a previous
            [FeatureOnlineStoreAdminService.ListFeatureViewSyncs][google.cloud.aiplatform.v1.FeatureOnlineStoreAdminService.ListFeatureViewSyncs]
            call. Provide this to retrieve the subsequent page.

            When paginating, all other parameters provided to
            [FeatureOnlineStoreAdminService.ListFeatureViewSyncs][google.cloud.aiplatform.v1.FeatureOnlineStoreAdminService.ListFeatureViewSyncs]
            must match the call that provided the page token.
        order_by (str):
            A comma-separated list of fields to order by, sorted in
            ascending order. Use "desc" after a field name for
            descending.

            Supported fields:

            - ``create_time``
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    filter: str = proto.Field(
        proto.STRING,
        number=2,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=3,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=4,
    )
    order_by: str = proto.Field(
        proto.STRING,
        number=5,
    )


class ListFeatureViewSyncsResponse(proto.Message):
    r"""Response message for
    [FeatureOnlineStoreAdminService.ListFeatureViewSyncs][google.cloud.aiplatform.v1.FeatureOnlineStoreAdminService.ListFeatureViewSyncs].

    Attributes:
        feature_view_syncs (MutableSequence[google.cloud.aiplatform_v1.types.FeatureViewSync]):
            The FeatureViewSyncs matching the request.
        next_page_token (str):
            A token, which can be sent as
            [ListFeatureViewSyncsRequest.page_token][google.cloud.aiplatform.v1.ListFeatureViewSyncsRequest.page_token]
            to retrieve the next page. If this field is omitted, there
            are no subsequent pages.
    """

    @property
    def raw_page(self):
        return self

    feature_view_syncs: MutableSequence[gca_feature_view_sync.FeatureViewSync] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message=gca_feature_view_sync.FeatureViewSync,
        )
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from typing import Dict, Literal, Optional, Union

from google.cloud.aiplatform.preview import datasets
from google.cloud.aiplatform.utils import _ipython_utils
from google.cloud.aiplatform_v1beta1.types import (
    tuning_job as gca_tuning_job_types,
)
from vertexai import generative_models
from vertexai.tuning import (
    SourceModel,
    TuningJob,
)


def train(
    *,
    source_model: Union[str, generative_models.GenerativeModel, SourceModel],
    train_dataset: Union[str, datasets.MultimodalDataset],
    validation_dataset: Optional[Union[str, datasets.MultimodalDataset]] = None,
    tuned_model_display_name: Optional[str] = None,
    tuning_mode: Optional[Literal["FULL", "PEFT_ADAPTER"]] = None,
    epochs: Optional[int] = None,
    learning_rate: Optional[float] = None,
    learning_rate_multiplier: Optional[float] = None,
    adapter_size: Optional[Literal[1, 4, 8, 16, 32]] = None,
    labels: Optional[Dict[str, str]] = None,
    output_uri: Optional[str] = None,
) -> "SupervisedTuningJob":
    """Tunes a model using supervised training.

    Args:
        source_model (str): Model name for tuning, e.g., "gemini-1.0-pro-002".
        train_dataset: Training dataset used for tuning. The dataset can be a JSONL file on Google Cloud Storage (specified as its GCS URI) or a Vertex Multimodal Dataset (either as the dataset object itself or as its resource name).
        validation_dataset: Validation dataset used for tuning. The dataset can be a JSONL file on Google Cloud Storage (specified as its GCS URI) or a Vertex Multimodal Dataset (either as the dataset object itself or as the resource name).
        tuned_model_display_name: The display name of the
          [TunedModel][google.cloud.aiplatform.v1.Model]. The name can be up to
          128 characters long and can consist of any UTF-8 characters.
        tuning_mode: Tuning mode for this tuning job. Can only be used with OSS
          models.
        epochs: Number of training epoches for this tuning job.
        learning_rate: Learning rate for tuning. Can only be used with OSS
          models. Mutually exclusive with `learning_rate_multiplier`.
        learning_rate_multiplier: Learning rate multiplier for tuning. Can only
          be used with 1P models. Mutually exclusive with `learning_rate`.
        adapter_size: Adapter size for tuning.
        labels: User-defined metadata to be associated with trained models
        output_uri: The Google Cloud Storage URI to write the tuned model to.
          Can only be used with OSS models.

    Returns:
        A `TuningJob` object.
    """
    if tuning_mode is None:
        tuning_mode_value = None
    elif tuning_mode == "FULL":
        tuning_mode_value = (
            gca_tuning_job_types.SupervisedTuningSpec.TuningMode.TUNING_MODE_FULL
        )
    elif tuning_mode == "PEFT_ADAPTER":
        tuning_mode_value = (
            gca_tuning_job_types.SupervisedTuningSpec.TuningMode.TUNING_MODE_PEFT_ADAPTER
        )
    else:
        raise ValueError(
            f"Unsupported tuning mode: {tuning_mode}. The supported tuning modes are [FULL, PEFT_ADAPTER]"
        )

    if learning_rate and learning_rate_multiplier:
        raise ValueError(
            "Only one of `learning_rate` and `learning_rate_multiplier` can be set."
        )

    if adapter_size is None:
        adapter_size_value = None
    elif adapter_size == 1:
        adapter_size_value = (
            gca_tuning_job_types.SupervisedHyperParameters.AdapterSize.ADAPTER_SIZE_ONE
        )
    elif adapter_size == 4:
        adapter_size_value = (
            gca_tuning_job_types.SupervisedHyperParameters.AdapterSize.ADAPTER_SIZE_FOUR
        )
    elif adapter_size == 8:
        adapter_size_value = (
            gca_tuning_job_types.SupervisedHyperParameters.AdapterSize.ADAPTER_SIZE_EIGHT
        )
    elif adapter_size == 16:
        adapter_size_value = (
            gca_tuning_job_types.SupervisedHyperParameters.AdapterSize.ADAPTER_SIZE_SIXTEEN
        )
    elif adapter_size == 32:
        adapter_size_value = (
            gca_tuning_job_types.SupervisedHyperParameters.AdapterSize.ADAPTER_SIZE_THIRTY_TWO
        )
    else:
        raise ValueError(
            f"Unsupported adapter size: {adapter_size}. The supported sizes are [1, 4, 8, 16, 32]"
        )
    if isinstance(train_dataset, datasets.MultimodalDataset):
        train_dataset = train_dataset.resource_name
    if isinstance(validation_dataset, datasets.MultimodalDataset):
        validation_dataset = validation_dataset.resource_name
    supervised_tuning_spec = gca_tuning_job_types.SupervisedTuningSpec(
        tuning_mode=tuning_mode_value,
        training_dataset_uri=train_dataset,
        validation_dataset_uri=validation_dataset,
        hyper_parameters=gca_tuning_job_types.SupervisedHyperParameters(
            epoch_count=epochs,
            learning_rate=learning_rate,
            learning_rate_multiplier=learning_rate_multiplier,
            adapter_size=adapter_size_value,
        ),
    )

    if isinstance(source_model, generative_models.GenerativeModel):
        source_model = source_model._prediction_resource_name.rpartition("/")[-1]

    if labels is None:
        labels = {}
    if "mg-source" not in labels and output_uri:
        labels["mg-source"] = "sdk"

    supervised_tuning_job = (
        SupervisedTuningJob._create(  # pylint: disable=protected-access
            base_model=source_model,
            tuning_spec=supervised_tuning_spec,
            tuned_model_display_name=tuned_model_display_name,
            labels=labels,
            output_uri=output_uri,
        )
    )
    _ipython_utils.display_model_tuning_button(supervised_tuning_job)

    return supervised_tuning_job


class SupervisedTuningJob(TuningJob):
    def __init__(self, tuning_job_name: str):
        super().__init__(tuning_job_name=tuning_job_name)
        _ipython_utils.display_model_tuning_button(self)



import os
from firecrawl import FirecrawlApp
from dotenv import load_dotenv
from pydantic import BaseModel, Field
from typing import List, Optional

load_dotenv()

class StockNewsArticle(BaseModel):
    title: str = Field(description="Article title")
    content: str = Field(description="Article content or summary")
    url: str = Field(description="Article URL")
    source: str = Field(description="News source")
    date: Optional[str] = Field(description="Publication date if available")

class StockNewsResponse(BaseModel):
    articles: List[StockNewsArticle] = Field(description="List of news articles found")
    total_count: int = Field(description="Total number of articles found")

class FirecrawlAgent:
    """Agent for performing stock news research using the Firecrawl API"""

    def __init__(self):
        self.api_key = os.getenv("FIRECRAWL_API_KEY")
        if not self.api_key:
            raise ValueError("Firecrawl API key not found in environment variables.")
        self.app = FirecrawlApp(api_key=self.api_key)

    def search(self, query: str):
        """Search for stock news and analysis from MarketWatch and Investing.com"""
        try:
            # Extract stock symbol from query
            symbol = query.split()[0] if query else "STOCK"
            
            # Create targeted URLs for financial news sites
            urls_to_search = [
                f"https://www.marketwatch.com/investing/stock/{symbol.lower()}",
                f"https://www.marketwatch.com/investing/stock/{symbol.lower()}/news",
                f"https://www.investing.com/search/?q={symbol}",
                f"https://finance.yahoo.com/quote/{symbol.upper()}/news/"
            ]
            
            # Create comprehensive prompt for financial news extraction
            prompt = f"""You are extracting financial news and analysis about stock {symbol}. 
            Extract ALL relevant news articles, analyst reports, and market commentary you can find.

            EXTRACTION INSTRUCTIONS:
            1. Find ALL news articles and analysis related to {symbol}
            2. For EACH article, extract:
               - title: Article headline (required)
               - content: Article summary or key points (required, max 500 chars)
               - url: Direct link to the article if available
               - source: News source name (MarketWatch, Investing.com, etc.)
               - date: Publication date if visible

            3. FOCUS ON:
               - Recent earnings reports
               - Analyst upgrades/downgrades
               - Company announcements
               - Market sentiment
               - Price target changes
               - Industry news affecting the stock

            4. RETURN FORMAT:
               - Extract at least 5-10 articles if available
               - Prioritize recent news (last 30 days)
               - Include both positive and negative sentiment
               - Focus on actionable investment insights

            Extract comprehensive financial news and analysis for investment decision making.
            """

            # Use Firecrawl extract method (like the real estate example)
            raw_response = self.app.extract(
                urls_to_search,
                prompt=prompt,
                schema=StockNewsResponse.model_json_schema()
            )
            
            # Process the response
            if hasattr(raw_response, 'success') and raw_response.success:
                # Handle Firecrawl response object
                articles = raw_response.data.get('articles', []) if hasattr(raw_response, 'data') else []
                
                # Convert to the expected format for the stock research agent
                scraped_results = []
                for article in articles:
                    scraped_results.append({
                        'content': article.get('content', ''),
                        'url': article.get('url', ''),
                        'metadata': {
                            'title': article.get('title', 'Financial News'),
                            'source': article.get('source', 'Financial News Site'),
                            'date': article.get('date', 'Recent')
                        }
                    })
                
                return scraped_results
                
            elif isinstance(raw_response, dict) and raw_response.get('success'):
                # Handle dictionary response
                articles = raw_response.get('data', {}).get('articles', [])
                
                # Convert to expected format
                scraped_results = []
                for article in articles:
                    scraped_results.append({
                        'content': article.get('content', ''),
                        'url': article.get('url', ''),
                        'metadata': {
                            'title': article.get('title', 'Financial News'),
                            'source': article.get('source', 'Financial News Site'),
                            'date': article.get('date', 'Recent')
                        }
                    })
                
                return scraped_results
            else:
                return {"error": f"Firecrawl extraction failed: {raw_response}"}
                
        except Exception as e:
            return {"error": f"Error during Firecrawl search: {e}"}




"""Pull Request creation agent with BMasterAI integration"""
import asyncio
import time
import uuid
from typing import Dict, Any, List, Optional
from utils.bmasterai_logging import get_logger, EventType, LogLevel
from utils.bmasterai_monitoring import get_monitor
from integrations.mcp_client import get_mcp_client
from utils.llm_client import get_llm_client

class PRCreatorAgent:
    """Agent for creating pull requests with improvements"""
    
    def __init__(self, agent_id: str = "pr_creator"):
        self.agent_id = agent_id
        self.agent_name = "PR Creator Agent"
        self.logger = get_logger()
        self.monitor = get_monitor()
        self.llm_client = get_llm_client()
        
        # Get model configuration
        from config import get_config_manager
        self.config_manager = get_config_manager()
        self.model_config = self.config_manager.get_model_config()
        self.model = self.model_config.pr_creator_model
        
        # Log agent initialization
        self.logger.log_agent_start(
            agent_id=self.agent_id,
            agent_name=self.agent_name,
            metadata={"initialized_at": time.time()}
        )
        
        # Track agent start in monitoring
        self.monitor.track_agent_start(self.agent_id)
    
    async def create_improvement_pr(self, repo_url: str, suggestions: List[Dict[str, Any]], 
                                  analysis_result: Dict[str, Any]) -> Dict[str, Any]:
        """Create a pull request with improvement suggestions"""
        task_id = str(uuid.uuid4())
        start_time = time.time()
        
        # Log task start
        self.logger.log_task_start(
            agent_id=self.agent_id,
            task_name="create_improvement_pr",
            task_id=task_id,
            metadata={
                "repo_url": repo_url,
                "suggestions_count": len(suggestions)
            }
        )
        
        try:
            # Step 1: Select high-priority suggestions for implementation
            selected_suggestions = self._select_implementable_suggestions(suggestions)
            
            if not selected_suggestions:
                return {
                    "success": False,
                    "error": "No implementable suggestions found",
                    "task_id": task_id
                }
            
            # Step 2: Generate branch name
            branch_name = self._generate_branch_name(selected_suggestions)
            
            # Step 3: Generate file changes for suggestions
            self.logger.log_event(
                agent_id=self.agent_id,
                event_type=EventType.TASK_START,
                message="Generating file changes",
                metadata={"step": "generate_changes", "task_id": task_id}
            )
            
            file_changes = await self._generate_file_changes(
                repo_url, selected_suggestions, analysis_result
            )
            
            # Step 4: Create pull request description
            pr_description = await self._generate_pr_description(
                selected_suggestions, analysis_result, file_changes
            )
            
            # Step 5: Use MCP to create the actual PR (mock implementation)
            pr_result = await self._create_pr_via_mcp(
                repo_url, branch_name, pr_description, file_changes
            )
            
            duration_ms = (time.time() - start_time) * 1000
            
            result = {
                "task_id": task_id,
                "branch_name": branch_name,
                "suggestions_implemented": selected_suggestions,
                "file_changes": file_changes,
                "pr_description": pr_description,
                "pr_result": pr_result,
                "summary": {
                    "changes_count": len(file_changes),
                    "suggestions_count": len(selected_suggestions),
                    "estimated_impact": self._calculate_impact(selected_suggestions)
                }
            }
            
            # Log successful completion
            self.logger.log_task_complete(
                agent_id=self.agent_id,
                task_name="create_improvement_pr",
                task_id=task_id,
                duration_ms=duration_ms,
                metadata={
                    "branch_name": branch_name,
                    "changes_count": len(file_changes),
                    "pr_number": pr_result.get("pull_request", {}).get("number")
                }
            )
            
            # Track performance metrics
            self.monitor.track_task_duration(self.agent_id, "create_improvement_pr", duration_ms)
            
            return {"success": True, "result": result}
            
        except Exception as e:
            duration_ms = (time.time() - start_time) * 1000
            
            # Log error
            self.logger.log_task_error(
                agent_id=self.agent_id,
                task_name="create_improvement_pr",
                task_id=task_id,
                error=str(e),
                duration_ms=duration_ms,
                metadata={"repo_url": repo_url}
            )
            
            # Track error
            self.monitor.track_error(self.agent_id, "pr_creation_error")
            
            return {"success": False, "error": str(e), "task_id": task_id}
    
    def _select_implementable_suggestions(self, suggestions: List[Dict[str, Any]]) -> List[Dict[str, Any]]:
        """Select suggestions that can be automatically implemented"""
        implementable_types = [
            "documentation",
            "security",
            "configuration",
            "gitignore",
            "readme"
        ]
        
        selected = []
        for suggestion in suggestions:
            category = suggestion.get("category", "").lower()
            priority = suggestion.get("priority", "").lower()
            
            # Select high-priority suggestions in implementable categories
            if priority == "high" and any(impl_type in category for impl_type in implementable_types):
                selected.append(suggestion)
            # Also select some medium-priority documentation improvements
            elif priority == "medium" and "documentation" in category:
                selected.append(suggestion)
        
        # Limit to 3 suggestions to keep PR manageable
        return selected[:3]
    
    def _generate_branch_name(self, suggestions: List[Dict[str, Any]]) -> str:
        """Generate a descriptive branch name for the PR"""
        timestamp = int(time.time())
        
        # Categorize suggestions
        categories = [s.get("category", "improvement") for s in suggestions]
        unique_categories = list(set(categories))
        
        if len(unique_categories) == 1:
            category_part = unique_categories[0]
        elif len(unique_categories) <= 2:
            category_part = "-".join(unique_categories)
        else:
            category_part = "improvements"
        
        return f"bmasterai/{category_part}-{timestamp}"
    
    async def _generate_file_changes(self, repo_url: str, suggestions: List[Dict[str, Any]], 
                                   analysis_result: Dict[str, Any]) -> List[Dict[str, Any]]:
        """Generate specific file changes for the suggestions"""
        file_changes = []
        
        for suggestion in suggestions:
            category = suggestion.get("category", "").lower()
            
            if "documentation" in category:
                changes = await self._generate_documentation_changes(repo_url, suggestion, analysis_result)
                file_changes.extend(changes)
            elif "security" in category:
                changes = await self._generate_security_changes(repo_url, suggestion, analysis_result)
                file_changes.extend(changes)
            elif "configuration" in category:
                changes = await self._generate_config_changes(repo_url, suggestion, analysis_result)
                file_changes.extend(changes)
        
        return file_changes
    
    async def _generate_documentation_changes(self, repo_url: str, suggestion: Dict[str, Any], 
                                            analysis_result: Dict[str, Any]) -> List[Dict[str, Any]]:
        """Generate documentation-related file changes"""
        changes = []
        
        # Check if README needs improvement
        readme_analysis = analysis_result.get("code_analysis", {}).get("key_files", {}).get("README.md")
        
        if not readme_analysis or not readme_analysis.get("exists", False):
            # Create new README
            readme_content = await self._generate_readme_content(repo_url, analysis_result)
            changes.append({
                "file_path": "README.md",
                "action": "create",
                "content": readme_content,
                "description": "Create comprehensive README documentation"
            })
        else:
            # Improve existing README
            improved_readme = await self._improve_readme_content(repo_url, analysis_result)
            changes.append({
                "file_path": "README.md", 
                "action": "update",
                "content": improved_readme,
                "description": "Enhance README with better documentation"
            })
        
        # Add CONTRIBUTING.md if missing
        if "contributing" in suggestion.get("description", "").lower():
            contributing_content = self._generate_contributing_content()
            changes.append({
                "file_path": "CONTRIBUTING.md",
                "action": "create",
                "content": contributing_content,
                "description": "Add contributing guidelines"
            })
        
        return changes
    
    async def _generate_readme_content(self, repo_url: str, analysis_result: Dict[str, Any]) -> str:
        """Generate comprehensive README content using LLM"""
        try:
            repo_info = analysis_result.get("repository_info", {})
            repo_name = repo_info.get("basic_info", {}).get("name", "Project")
            description = repo_info.get("basic_info", {}).get("description", "")
            languages = repo_info.get("languages", {})
            
            prompt = f"""
            Create a comprehensive README.md for a GitHub repository with the following information:

            Repository Name: {repo_name}
            Description: {description}
            Main Languages: {', '.join(languages.keys())}
            
            Include these sections:
            1. Project title and description
            2. Installation instructions
            3. Usage examples
            4. Features
            5. Contributing guidelines
            6. License information
            
            Make it professional, clear, and helpful for new users.
            """
            
            response = await self.llm_client.call_llm(
                model=self.model,
                prompt=prompt,
                max_tokens=1500
            )
            
            return response.get("response", self._generate_fallback_readme(repo_name, description))
            
        except Exception as e:
            self.logger.log_event(
                agent_id=self.agent_id,
                event_type=EventType.TASK_ERROR,
                message=f"Failed to generate README: {str(e)}",
                level=LogLevel.ERROR
            )
            return self._generate_fallback_readme(repo_name, description)
    
    def _generate_fallback_readme(self, repo_name: str, description: str) -> str:
        """Generate a basic README as fallback"""
        return f"""# {repo_name}

{description or "A software project"}

## Installation

```bash
# Add installation instructions here
```

## Usage

```bash
# Add usage examples here
```

## Features

- List key features
- Add more features

## Contributing

Contributions are welcome! Please feel free to submit a Pull Request.

## License

This project is licensed under the MIT License - see the LICENSE file for details.
"""
    
    async def _generate_security_changes(self, repo_url: str, suggestion: Dict[str, Any], 
                                       analysis_result: Dict[str, Any]) -> List[Dict[str, Any]]:
        """Generate security-related file changes"""
        changes = []
        
        # Create or update .gitignore
        gitignore_content = self._generate_gitignore_content(analysis_result)
        changes.append({
            "file_path": ".gitignore",
            "action": "create_or_update",
            "content": gitignore_content,
            "description": "Add comprehensive .gitignore to prevent sensitive files"
        })
        
        # Create .env.example if environment variables are detected
        if "secret" in suggestion.get("description", "").lower():
            env_example_content = self._generate_env_example_content()
            changes.append({
                "file_path": ".env.example",
                "action": "create",
                "content": env_example_content,
                "description": "Add environment variables template"
            })
        
        return changes
    
    def _generate_gitignore_content(self, analysis_result: Dict[str, Any]) -> str:
        """Generate comprehensive .gitignore content"""
        languages = analysis_result.get("repository_info", {}).get("languages", {})
        
        gitignore_sections = [
            "# Environment variables",
            ".env",
            ".env.local",
            ".env.*.local",
            "",
            "# Dependencies",
            "node_modules/",
            "__pycache__/",
            "*.pyc",
            "",
            "# IDE",
            ".vscode/",
            ".idea/",
            "*.swp",
            "*.swo",
            "",
            "# Logs",
            "*.log",
            "logs/",
            "",
            "# OS generated files",
            ".DS_Store",
            "Thumbs.db"
        ]
        
        # Add language-specific ignores
        if "Python" in languages:
            gitignore_sections.extend([
                "",
                "# Python",
                "*.egg-info/",
                "dist/",
                "build/",
                ".pytest_cache/",
                ".coverage"
            ])
        
        if "JavaScript" in languages or "TypeScript" in languages:
            gitignore_sections.extend([
                "",
                "# Node.js",
                "npm-debug.log*",
                "yarn-debug.log*",
                "yarn-error.log*",
                ".npm",
                ".yarn-integrity"
            ])
        
        return "\n".join(gitignore_sections)
    
    def _generate_env_example_content(self) -> str:
        """Generate .env.example template"""
        return """# Environment Variables Template
# Copy this file to .env and fill in your actual values

# API Keys
API_KEY=your_api_key_here
SECRET_KEY=your_secret_key_here

# Database
DATABASE_URL=your_database_url_here

# Application Settings
DEBUG=false
PORT=3000
"""
    
    async def _generate_config_changes(self, repo_url: str, suggestion: Dict[str, Any], 
                                     analysis_result: Dict[str, Any]) -> List[Dict[str, Any]]:
        """Generate configuration-related file changes"""
        changes = []
        
        # Add EditorConfig for consistent formatting
        editorconfig_content = self._generate_editorconfig_content()
        changes.append({
            "file_path": ".editorconfig",
            "action": "create",
            "content": editorconfig_content,
            "description": "Add EditorConfig for consistent code formatting"
        })
        
        return changes
    
    def _generate_editorconfig_content(self) -> str:
        """Generate .editorconfig content"""
        return """root = true

[*]
indent_style = space
indent_size = 2
end_of_line = lf
charset = utf-8
trim_trailing_whitespace = true
insert_final_newline = true

[*.py]
indent_size = 4

[*.md]
trim_trailing_whitespace = false
"""
    
    def _generate_contributing_content(self) -> str:
        """Generate CONTRIBUTING.md content"""
        return """# Contributing

We love your input! We want to make contributing to this project as easy and transparent as possible.

## Development Process

1. Fork the repo and create your branch from `main`
2. If you've added code that should be tested, add tests
3. Ensure the test suite passes
4. Make sure your code lints
5. Issue that pull request!

## Pull Request Process

1. Update the README.md with details of changes to the interface
2. Increase the version numbers in any examples files and the README.md
3. You may merge the Pull Request in once you have the sign-off of two other developers

## Code of Conduct

By participating, you are expected to uphold our code of conduct.

## License

By contributing, you agree that your contributions will be licensed under the same license as the project.
"""
    
    async def _generate_pr_description(self, suggestions: List[Dict[str, Any]], 
                                     analysis_result: Dict[str, Any], 
                                     file_changes: List[Dict[str, Any]]) -> str:
        """Generate comprehensive PR description"""
        try:
            repo_name = analysis_result.get("repository_info", {}).get("basic_info", {}).get("name", "Repository")
            
            changes_summary = []
            for change in file_changes:
                changes_summary.append(f"- {change['action'].title()} {change['file_path']}: {change['description']}")
            
            prompt = f"""
            Create a professional GitHub Pull Request description for improvements to {repo_name}.

            Implemented Suggestions:
            {chr(10).join([f"- {s.get('title', '')}: {s.get('description', '')}" for s in suggestions])}

            File Changes:
            {chr(10).join(changes_summary)}

            Include:
            1. Clear title and overview
            2. What was changed and why
            3. Benefits of these changes
            4. Any breaking changes (if applicable)
            5. Checklist for review

            Make it professional and easy to review.
            """
            
            response = await self.llm_client.call_llm(
                model=self.model,
                prompt=prompt,
                max_tokens=1000
            )
            
            return response.get("response", self._generate_fallback_pr_description(suggestions, file_changes))
            
        except Exception as e:
            self.logger.log_event(
                agent_id=self.agent_id,
                event_type=EventType.TASK_ERROR,
                message=f"Failed to generate PR description: {str(e)}",
                level=LogLevel.ERROR
            )
            return self._generate_fallback_pr_description(suggestions, file_changes)
    
    def _generate_fallback_pr_description(self, suggestions: List[Dict[str, Any]], 
                                        file_changes: List[Dict[str, Any]]) -> str:
        """Generate fallback PR description"""
        changes_list = "\n".join([f"- {change['description']}" for change in file_changes])
        suggestions_list = "\n".join([f"- {s.get('title', 'Improvement')}" for s in suggestions])
        
        return f"""# Repository Improvements

This PR implements several automated improvements to enhance the repository's quality, security, and maintainability.

## Changes Made

{changes_list}

## Implemented Suggestions

{suggestions_list}

## Benefits

- Improved documentation for better onboarding
- Enhanced security practices
- Better code organization and consistency
- Clearer project structure

## Review Checklist

- [ ] Documentation is clear and comprehensive
- [ ] Security improvements are appropriate
- [ ] No sensitive information is exposed
- [ ] Changes follow project conventions

---

*This PR was generated automatically by BMasterAI GitHub Analyzer*
"""
    
    async def _create_pr_via_mcp(self, repo_url: str, branch_name: str, 
                               description: str, file_changes: List[Dict[str, Any]]) -> Dict[str, Any]:
        """Create pull request via MCP client"""
        try:
            async with get_mcp_client() as mcp_client:
                # Clone repository
                repo_path = f"/tmp/{branch_name}"
                await mcp_client.clone_repository(repo_url, repo_path)
                
                # Create new branch
                await mcp_client.create_branch(repo_path, branch_name)
                
                # Apply file changes
                modified_files = []
                for change in file_changes:
                    await mcp_client.write_file(repo_path, change["file_path"], change["content"])
                    modified_files.append(change["file_path"])
                
                # Commit changes
                commit_message = f"Automated improvements: {', '.join([c['description'] for c in file_changes[:2]])}"
                await mcp_client.commit_changes(repo_path, commit_message, modified_files)
                
                # Create pull request
                pr_title = f"🤖 Automated Repository Improvements ({branch_name.split('/')[-1]})"
                pr_result = await mcp_client.create_pull_request(
                    repo_path=repo_path,
                    title=pr_title,
                    description=description,
                    head_branch=branch_name,
                    base_branch="main"
                )
                
                return pr_result
                
        except Exception as e:
            self.logger.log_event(
                agent_id=self.agent_id,
                event_type=EventType.TASK_ERROR,
                message=f"MCP PR creation failed: {str(e)}",
                level=LogLevel.ERROR
            )
            
            # Return mock result for demonstration
            return {
                "success": True,
                "pull_request": {
                    "number": 42,
                    "title": f"🤖 Automated Repository Improvements ({branch_name.split('/')[-1]})",
                    "url": f"https://github.com/mock/repo/pull/42",
                    "branch": branch_name,
                    "status": "open"
                },
                "message": "Pull request created successfully (mock)"
            }
    
    def _calculate_impact(self, suggestions: List[Dict[str, Any]]) -> str:
        """Calculate estimated impact of suggestions"""
        high_priority_count = len([s for s in suggestions if s.get("priority") == "High"])
        medium_priority_count = len([s for s in suggestions if s.get("priority") == "Medium"])
        
        if high_priority_count >= 2:
            return "High - Significant improvements to security and documentation"
        elif high_priority_count >= 1 or medium_priority_count >= 2:
            return "Medium - Notable improvements to repository quality"
        else:
            return "Low - Minor improvements and cleanup"
    
    async def create_feature_pr(self, repo_url: str, branch_name: str, base_branch: str, 
                               feature_description: str, code_generation_result: Dict[str, Any]) -> Dict[str, Any]:
        """Create a pull request for a new feature"""
        task_id = str(uuid.uuid4())
        start_time = time.time()
        
        # Log task start
        self.logger.log_task_start(
            agent_id=self.agent_id,
            task_name="create_feature_pr",
            task_id=task_id,
            metadata={
                "repo_url": repo_url,
                "branch_name": branch_name,
                "feature_description": feature_description
            }
        )
        
        try:
            # Generate PR description for feature
            pr_description = await self._generate_feature_pr_description(
                feature_description, code_generation_result
            )
            
            # Create pull request via MCP
            pr_result = await self._create_feature_pr_via_mcp(
                repo_url, branch_name, base_branch, pr_description, code_generation_result
            )
            
            duration_ms = (time.time() - start_time) * 1000
            
            result = {
                "task_id": task_id,
                "branch_name": branch_name,
                "base_branch": base_branch,
                "feature_description": feature_description,
                "pr_description": pr_description,
                "pr_result": pr_result,
                "files_modified": len(code_generation_result.get("files", [])),
                "summary": {
                    "feature_implemented": True,
                    "files_count": len(code_generation_result.get("files", [])),
                    "estimated_impact": "Feature implementation"
                }
            }
            
            # Log successful completion
            self.logger.log_task_complete(
                agent_id=self.agent_id,
                task_name="create_feature_pr",
                task_id=task_id,
                duration_ms=duration_ms,
                metadata={
                    "branch_name": branch_name,
                    "files_count": len(code_generation_result.get("files", [])),
                    "pr_number": pr_result.get("pull_request", {}).get("number")
                }
            )
            
            # Track performance metrics
            self.monitor.track_task_duration(self.agent_id, "create_feature_pr", duration_ms)
            
            return {"success": True, "result": result}
            
        except Exception as e:
            duration_ms = (time.time() - start_time) * 1000
            
            # Log error
            self.logger.log_task_error(
                agent_id=self.agent_id,
                task_name="create_feature_pr",
                task_id=task_id,
                error=str(e),
                duration_ms=duration_ms,
                metadata={"repo_url": repo_url, "branch_name": branch_name}
            )
            
            # Track error
            self.monitor.track_error(self.agent_id, "feature_pr_creation_error")
            
            return {"success": False, "error": str(e), "task_id": task_id}
    
    async def _generate_feature_pr_description(self, feature_description: str, 
                                             code_generation_result: Dict[str, Any]) -> str:
        """Generate PR description for feature implementation"""
        try:
            files = code_generation_result.get("files", [])
            files_summary = []
            for file_info in files:
                files_summary.append(f"- {file_info.get('path', 'Unknown')}: {file_info.get('description', 'Modified')}")
            
            prompt = f"""
            Create a professional GitHub Pull Request description for a new feature implementation.

            Feature Description: {feature_description}

            Files Modified:
            {chr(10).join(files_summary)}

            Include:
            1. Clear feature overview
            2. Implementation details
            3. Files changed and why
            4. Testing recommendations
            5. Review checklist

            Make it professional and easy to review.
            """
            
            response = await self.llm_client.call_llm(
                model=self.model,
                prompt=prompt,
                max_tokens=1000
            )
            
            return response.get("response", self._generate_fallback_feature_pr_description(feature_description, files))
            
        except Exception as e:
            self.logger.log_event(
                agent_id=self.agent_id,
                event_type=EventType.TASK_ERROR,
                message=f"Failed to generate feature PR description: {str(e)}",
                level=LogLevel.ERROR
            )
            return self._generate_fallback_feature_pr_description(feature_description, files)
    
    def _generate_fallback_feature_pr_description(self, feature_description: str, 
                                                files: List[Dict[str, Any]]) -> str:
        """Generate fallback feature PR description"""
        files_list = "\n".join([f"- {f.get('path', 'Unknown file')}" for f in files])
        
        return f"""# Feature Implementation: {feature_description}

## Overview

This PR implements the requested feature: {feature_description}

## Files Modified

{files_list}

## Implementation Details

- Feature has been implemented according to specifications
- Code follows project conventions and best practices
- All changes are focused on the specific feature requirements

## Testing

- [ ] Feature functionality has been tested
- [ ] No existing functionality is broken
- [ ] Edge cases have been considered

## Review Checklist

- [ ] Code follows project style guidelines
- [ ] Feature works as expected
- [ ] No security vulnerabilities introduced
- [ ] Documentation updated if needed

---

*This PR was generated automatically by BMasterAI Feature Agent*
"""
    
    async def _create_feature_pr_via_mcp(self, repo_url: str, branch_name: str, base_branch: str,
                                       description: str, code_generation_result: Dict[str, Any]) -> Dict[str, Any]:
        """Create feature pull request via MCP client"""
        try:
            # For now, return a mock result since the actual MCP implementation would be complex
            return {
                "success": True,
                "pull_request": {
                    "number": 43,
                    "title": f"🚀 Feature: {code_generation_result.get('feature_name', 'New Feature')}",
                    "url": f"https://github.com/mock/repo/pull/43",
                    "branch": branch_name,
                    "base_branch": base_branch,
                    "status": "open"
                },
                "message": "Feature pull request created successfully (mock)"
            }
                
        except Exception as e:
            self.logger.log_event(
                agent_id=self.agent_id,
                event_type=EventType.TASK_ERROR,
                message=f"MCP feature PR creation failed: {str(e)}",
                level=LogLevel.ERROR
            )
            
            # Return mock result for demonstration
            return {
                "success": True,
                "pull_request": {
                    "number": 43,
                    "title": f"🚀 Feature: {code_generation_result.get('feature_name', 'New Feature')}",
                    "url": f"https://github.com/mock/repo/pull/43",
                    "branch": branch_name,
                    "base_branch": base_branch,
                    "status": "open"
                },
                "message": "Feature pull request created successfully (mock)"
            }

    def stop(self):
        """Stop the agent and clean up resources"""
        self.logger.log_agent_stop(
            agent_id=self.agent_id,
            agent_name=self.agent_name,
            metadata={"stopped_at": time.time()}
        )
        
        self.monitor.track_agent_stop(self.agent_id)

def get_pr_creator() -> PRCreatorAgent:
    """Get PR creator agent instance"""
    return PRCreatorAgent()



# -*- coding: utf-8 -*-

# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from google.cloud.aiplatform_v1beta1.types import (
    evaluation_service as gapic_eval_service_types,
)
from vertexai.preview.evaluation import utils
from vertexai.preview.evaluation.metrics import (
    _base as metrics_base,
)
from vertexai.preview.evaluation.metrics import (
    _default_templates,
)
from vertexai.preview.evaluation.metrics import (
    custom_output_config,
)
from vertexai.preview.evaluation.metrics import pairwise_metric
from vertexai.preview.evaluation.metrics import pointwise_metric
from vertexai.preview.evaluation.metrics import (
    rubric_based_metric,
)


AutoraterConfig = gapic_eval_service_types.AutoraterConfig

_POINTWISE_OUTPUT_CONFIG = custom_output_config.CustomOutputConfig(
    return_raw_output=True,
    parsing_fn=utils.parse_pointwise_rubric_result,
)

_PAIRWISE_OUTPUT_CONFIG = custom_output_config.CustomOutputConfig(
    return_raw_output=True,
    parsing_fn=utils.parse_pairwise_rubric_result,
)
_PAIRWISE_AUTORATER_CONFIG = AutoraterConfig(
    sampling_count=1,
)


class PredefinedRubricMetrics:
    """Predefined rubric-based metrics."""

    class Pointwise:
        """Pointwise rubric-based metrics."""

        INSTRUCTION_FOLLOWING = rubric_based_metric.RubricBasedMetric(
            generation_config=metrics_base.RubricGenerationConfig(
                prompt_template=_default_templates.INSTRUCTION_FOLLOWING_RUBRIC_GENERATION_PROMPT_TEMPLATE,
            ),
            critique_metric=pointwise_metric.PointwiseMetric(
                metric="rb_instruction_following",
                metric_prompt_template=_default_templates.INSTRUCTION_FOLLOWING_RUBRIC_CRITIQUE_TEMPLATE,
                custom_output_config=_POINTWISE_OUTPUT_CONFIG,
            ),
        )
        MULTIMODAL_UNDERSTANDING = rubric_based_metric.RubricBasedMetric(
            generation_config=metrics_base.RubricGenerationConfig(
                prompt_template=_default_templates.MULTIMODAL_UNDERSTANDING_RUBRIC_GENERATION_PROMPT_TEMPLATE
            ),
            critique_metric=pointwise_metric.PointwiseMetric(
                metric="rb_multimodal_understanding",
                metric_prompt_template=_default_templates.MULTIMODAL_UNDERSTANDING_RUBRIC_CRITIQUE_TEMPLATE,
                custom_output_config=_POINTWISE_OUTPUT_CONFIG,
            ),
        )
        TEXT_QUALITY = rubric_based_metric.RubricBasedMetric(
            generation_config=metrics_base.RubricGenerationConfig(
                prompt_template=_default_templates.TEXT_QUALITY_RUBRIC_GENERATION_PROMPT_TEMPLATE
            ),
            critique_metric=pointwise_metric.PointwiseMetric(
                metric="rb_text_quality",
                metric_prompt_template=_default_templates.TEXT_QUALITY_RUBRIC_CRITIQUE_TEMPLATE,
                custom_output_config=_POINTWISE_OUTPUT_CONFIG,
            ),
        )

    class Pairwise:
        """Pairwise rubric-based metrics."""

        INSTRUCTION_FOLLOWING = rubric_based_metric.RubricBasedMetric(
            generation_config=metrics_base.RubricGenerationConfig(
                prompt_template=_default_templates.INSTRUCTION_FOLLOWING_RUBRIC_GENERATION_PROMPT_TEMPLATE,
            ),
            critique_metric=pairwise_metric.PairwiseMetric(
                metric="pairwise_rb_instruction_following",
                metric_prompt_template=_default_templates.PAIRWISE_INSTRUCTION_FOLLOWING_RUBRIC_CRITIQUE_TEMPLATE,
                custom_output_config=_PAIRWISE_OUTPUT_CONFIG,
                autorater_config=_PAIRWISE_AUTORATER_CONFIG,
            ),
        )
        MULTIMODAL_UNDERSTANDING = rubric_based_metric.RubricBasedMetric(
            generation_config=metrics_base.RubricGenerationConfig(
                prompt_template=_default_templates.MULTIMODAL_UNDERSTANDING_RUBRIC_GENERATION_PROMPT_TEMPLATE
            ),
            critique_metric=pairwise_metric.PairwiseMetric(
                metric="pairwise_rb_multimodal_understanding",
                metric_prompt_template=_default_templates.PAIRWISE_MULTIMODAL_UNDERSTANDING_RUBRIC_CRITIQUE_TEMPLATE,
                custom_output_config=_PAIRWISE_OUTPUT_CONFIG,
                autorater_config=_PAIRWISE_AUTORATER_CONFIG,
            ),
        )
        TEXT_QUALITY = rubric_based_metric.RubricBasedMetric(
            generation_config=metrics_base.RubricGenerationConfig(
                prompt_template=_default_templates.TEXT_QUALITY_RUBRIC_GENERATION_PROMPT_TEMPLATE
            ),
            critique_metric=pairwise_metric.PairwiseMetric(
                metric="pairwise_rb_text_quality",
                metric_prompt_template=_default_templates.PAIRWISE_TEXT_QUALITY_RUBRIC_CRITIQUE_TEMPLATE,
                custom_output_config=_PAIRWISE_OUTPUT_CONFIG,
                autorater_config=_PAIRWISE_AUTORATER_CONFIG,
            ),
        )



# Copyright 2017 Google LLC All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Types exported from this package."""

from google.cloud.spanner_v1 import Type
from google.cloud.spanner_v1 import TypeAnnotationCode
from google.cloud.spanner_v1 import TypeCode
from google.cloud.spanner_v1 import StructType
from google.protobuf.message import Message
from google.protobuf.internal.enum_type_wrapper import EnumTypeWrapper


# Scalar parameter types
STRING = Type(code=TypeCode.STRING)
BYTES = Type(code=TypeCode.BYTES)
BOOL = Type(code=TypeCode.BOOL)
INT64 = Type(code=TypeCode.INT64)
FLOAT64 = Type(code=TypeCode.FLOAT64)
FLOAT32 = Type(code=TypeCode.FLOAT32)
DATE = Type(code=TypeCode.DATE)
TIMESTAMP = Type(code=TypeCode.TIMESTAMP)
NUMERIC = Type(code=TypeCode.NUMERIC)
JSON = Type(code=TypeCode.JSON)
PG_NUMERIC = Type(code=TypeCode.NUMERIC, type_annotation=TypeAnnotationCode.PG_NUMERIC)
PG_JSONB = Type(code=TypeCode.JSON, type_annotation=TypeAnnotationCode.PG_JSONB)
PG_OID = Type(code=TypeCode.INT64, type_annotation=TypeAnnotationCode.PG_OID)
INTERVAL = Type(code=TypeCode.INTERVAL)


def Array(element_type):
    """Construct an array parameter type description protobuf.

    :type element_type: :class:`~google.cloud.spanner_v1.types.Type`
    :param element_type: the type of elements of the array

    :rtype: :class:`google.cloud.spanner_v1.types.Type`
    :returns: the appropriate array-type protobuf
    """
    return Type(code=TypeCode.ARRAY, array_element_type=element_type)


def StructField(name, field_type):
    """Construct a field description protobuf.

    :type name: str
    :param name: the name of the field

    :type field_type: :class:`google.cloud.spanner_v1.types.Type`
    :param field_type: the type of the field

    :rtype: :class:`google.cloud.spanner_v1.types.StructType.Field`
    :returns: the appropriate struct-field-type protobuf
    """
    return StructType.Field(name=name, type_=field_type)


def Struct(fields):
    """Construct a struct parameter type description protobuf.

    :type fields: list of :class:`google.cloud.spanner_v1.types.StructType.Field`
    :param fields: the fields of the struct

    :rtype: :class:`type_pb2.Type`
    :returns: the appropriate struct-type protobuf
    """
    return Type(code=TypeCode.STRUCT, struct_type=StructType(fields=fields))


def ProtoMessage(proto_message_object):
    """Construct a proto message type description protobuf.

    :type proto_message_object: :class:`google.protobuf.message.Message`
    :param proto_message_object: the proto message instance

    :rtype: :class:`type_pb2.Type`
    :returns: the appropriate proto-message-type protobuf
    """
    if not isinstance(proto_message_object, Message):
        raise ValueError("Expected input object of type Proto Message.")
    return Type(
        code=TypeCode.PROTO, proto_type_fqn=proto_message_object.DESCRIPTOR.full_name
    )


def ProtoEnum(proto_enum_object):
    """Construct a proto enum type description protobuf.

    :type proto_enum_object: :class:`google.protobuf.internal.enum_type_wrapper.EnumTypeWrapper`
    :param proto_enum_object: the proto enum instance

    :rtype: :class:`type_pb2.Type`
    :returns: the appropriate proto-enum-type protobuf
    """
    if not isinstance(proto_enum_object, EnumTypeWrapper):
        raise ValueError("Expected input object of type Proto Enum")
    return Type(
        code=TypeCode.ENUM, proto_type_fqn=proto_enum_object.DESCRIPTOR.full_name
    )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["FunctionShellTool"]


class FunctionShellTool(BaseModel):
    """A tool that allows the model to execute shell commands."""

    type: Literal["shell"]
    """The type of the shell tool. Always `shell`."""



# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from __future__ import annotations

from typing import Sequence, TYPE_CHECKING

from google.cloud.bigtable_v2.types import ReadRowsRequest as ReadRowsRequestPB
from google.cloud.bigtable_v2.types import ReadRowsResponse as ReadRowsResponsePB
from google.cloud.bigtable_v2.types import RowSet as RowSetPB
from google.cloud.bigtable_v2.types import RowRange as RowRangePB

from google.cloud.bigtable.data.row import Row, Cell
from google.cloud.bigtable.data.read_rows_query import ReadRowsQuery
from google.cloud.bigtable.data.exceptions import InvalidChunk
from google.cloud.bigtable.data.exceptions import _RowSetComplete
from google.cloud.bigtable.data.exceptions import _ResetRow
from google.cloud.bigtable.data._helpers import _attempt_timeout_generator
from google.cloud.bigtable.data._helpers import _retry_exception_factory

from google.api_core import retry as retries
from google.api_core.retry import exponential_sleep_generator

from google.cloud.bigtable.data._cross_sync import CrossSync

if TYPE_CHECKING:
    if CrossSync.is_async:
        from google.cloud.bigtable.data._async.client import (
            _DataApiTargetAsync as TargetType,
        )
    else:
        from google.cloud.bigtable.data._sync_autogen.client import _DataApiTarget as TargetType  # type: ignore

__CROSS_SYNC_OUTPUT__ = "google.cloud.bigtable.data._sync_autogen._read_rows"


@CrossSync.convert_class("_ReadRowsOperation")
class _ReadRowsOperationAsync:
    """
    ReadRowsOperation handles the logic of merging chunks from a ReadRowsResponse stream
    into a stream of Row objects.

    ReadRowsOperation.merge_row_response_stream takes in a stream of ReadRowsResponse
    and turns them into a stream of Row objects using an internal
    StateMachine.

    ReadRowsOperation(request, client) handles row merging logic end-to-end, including
    performing retries on stream errors.

    Args:
        query: The query to execute
        target: The table or view to send the request to
        operation_timeout: The total time to allow for the operation, in seconds
        attempt_timeout: The time to allow for each individual attempt, in seconds
        retryable_exceptions: A list of exceptions that should trigger a retry
    """

    __slots__ = (
        "attempt_timeout_gen",
        "operation_timeout",
        "request",
        "target",
        "_predicate",
        "_last_yielded_row_key",
        "_remaining_count",
    )

    def __init__(
        self,
        query: ReadRowsQuery,
        target: TargetType,
        operation_timeout: float,
        attempt_timeout: float,
        retryable_exceptions: Sequence[type[Exception]] = (),
    ):
        self.attempt_timeout_gen = _attempt_timeout_generator(
            attempt_timeout, operation_timeout
        )
        self.operation_timeout = operation_timeout
        if isinstance(query, dict):
            self.request = ReadRowsRequestPB(
                **query,
                **target._request_path,
                app_profile_id=target.app_profile_id,
            )
        else:
            self.request = query._to_pb(target)
        self.target = target
        self._predicate = retries.if_exception_type(*retryable_exceptions)
        self._last_yielded_row_key: bytes | None = None
        self._remaining_count: int | None = self.request.rows_limit or None

    def start_operation(self) -> CrossSync.Iterable[Row]:
        """
        Start the read_rows operation, retrying on retryable errors.

        Yields:
            Row: The next row in the stream
        """
        return CrossSync.retry_target_stream(
            self._read_rows_attempt,
            self._predicate,
            exponential_sleep_generator(0.01, 60, multiplier=2),
            self.operation_timeout,
            exception_factory=_retry_exception_factory,
        )

    def _read_rows_attempt(self) -> CrossSync.Iterable[Row]:
        """
        Attempt a single read_rows rpc call.
        This function is intended to be wrapped by retry logic,
        which will call this function until it succeeds or
        a non-retryable error is raised.

        Yields:
            Row: The next row in the stream
        """
        # revise request keys and ranges between attempts
        if self._last_yielded_row_key is not None:
            # if this is a retry, try to trim down the request to avoid ones we've already processed
            try:
                self.request.rows = self._revise_request_rowset(
                    row_set=self.request.rows,
                    last_seen_row_key=self._last_yielded_row_key,
                )
            except _RowSetComplete:
                # if we've already seen all the rows, we're done
                return self.merge_rows(None)
        # revise the limit based on number of rows already yielded
        if self._remaining_count is not None:
            self.request.rows_limit = self._remaining_count
            if self._remaining_count == 0:
                return self.merge_rows(None)
        # create and return a new row merger
        gapic_stream = self.target.client._gapic_client.read_rows(
            self.request,
            timeout=next(self.attempt_timeout_gen),
            retry=None,
        )
        chunked_stream = self.chunk_stream(gapic_stream)
        return self.merge_rows(chunked_stream)

    @CrossSync.convert()
    async def chunk_stream(
        self, stream: CrossSync.Awaitable[CrossSync.Iterable[ReadRowsResponsePB]]
    ) -> CrossSync.Iterable[ReadRowsResponsePB.CellChunk]:
        """
        process chunks out of raw read_rows stream

        Args:
            stream: the raw read_rows stream from the gapic client
        Yields:
            ReadRowsResponsePB.CellChunk: the next chunk in the stream
        """
        async for resp in await stream:
            # extract proto from proto-plus wrapper
            resp = resp._pb

            # handle last_scanned_row_key packets, sent when server
            # has scanned past the end of the row range
            if resp.last_scanned_row_key:
                if (
                    self._last_yielded_row_key is not None
                    and resp.last_scanned_row_key <= self._last_yielded_row_key
                ):
                    raise InvalidChunk("last scanned out of order")
                self._last_yielded_row_key = resp.last_scanned_row_key

            current_key = None
            # process each chunk in the response
            for c in resp.chunks:
                if current_key is None:
                    current_key = c.row_key
                    if current_key is None:
                        raise InvalidChunk("first chunk is missing a row key")
                    elif (
                        self._last_yielded_row_key
                        and current_key <= self._last_yielded_row_key
                    ):
                        raise InvalidChunk("row keys should be strictly increasing")

                yield c

                if c.reset_row:
                    current_key = None
                elif c.commit_row:
                    # update row state after each commit
                    self._last_yielded_row_key = current_key
                    if self._remaining_count is not None:
                        self._remaining_count -= 1
                        if self._remaining_count < 0:
                            raise InvalidChunk("emit count exceeds row limit")
                    current_key = None

    @staticmethod
    @CrossSync.convert(
        replace_symbols={"__aiter__": "__iter__", "__anext__": "__next__"},
    )
    async def merge_rows(
        chunks: CrossSync.Iterable[ReadRowsResponsePB.CellChunk] | None,
    ) -> CrossSync.Iterable[Row]:
        """
        Merge chunks into rows

        Args:
            chunks: the chunk stream to merge
        Yields:
            Row: the next row in the stream
        """
        if chunks is None:
            return
        it = chunks.__aiter__()
        # For each row
        while True:
            try:
                c = await it.__anext__()
            except CrossSync.StopIteration:
                # stream complete
                return
            row_key = c.row_key

            if not row_key:
                raise InvalidChunk("first row chunk is missing key")

            cells = []

            # shared per cell storage
            family: str | None = None
            qualifier: bytes | None = None

            try:
                # for each cell
                while True:
                    if c.reset_row:
                        raise _ResetRow(c)
                    k = c.row_key
                    f = c.family_name.value
                    q = c.qualifier.value if c.HasField("qualifier") else None
                    if k and k != row_key:
                        raise InvalidChunk("unexpected new row key")
                    if f:
                        family = f
                        if q is not None:
                            qualifier = q
                        else:
                            raise InvalidChunk("new family without qualifier")
                    elif family is None:
                        raise InvalidChunk("missing family")
                    elif q is not None:
                        if family is None:
                            raise InvalidChunk("new qualifier without family")
                        qualifier = q
                    elif qualifier is None:
                        raise InvalidChunk("missing qualifier")

                    ts = c.timestamp_micros
                    labels = c.labels if c.labels else []
                    value = c.value

                    # merge split cells
                    if c.value_size > 0:
                        buffer = [value]
                        while c.value_size > 0:
                            # throws when premature end
                            c = await it.__anext__()

                            t = c.timestamp_micros
                            cl = c.labels
                            k = c.row_key
                            if (
                                c.HasField("family_name")
                                and c.family_name.value != family
                            ):
                                raise InvalidChunk("family changed mid cell")
                            if (
                                c.HasField("qualifier")
                                and c.qualifier.value != qualifier
                            ):
                                raise InvalidChunk("qualifier changed mid cell")
                            if t and t != ts:
                                raise InvalidChunk("timestamp changed mid cell")
                            if cl and cl != labels:
                                raise InvalidChunk("labels changed mid cell")
                            if k and k != row_key:
                                raise InvalidChunk("row key changed mid cell")

                            if c.reset_row:
                                raise _ResetRow(c)
                            buffer.append(c.value)
                        value = b"".join(buffer)
                    cells.append(
                        Cell(value, row_key, family, qualifier, ts, list(labels))
                    )
                    if c.commit_row:
                        yield Row(row_key, cells)
                        break
                    c = await it.__anext__()
            except _ResetRow as e:
                c = e.chunk
                if (
                    c.row_key
                    or c.HasField("family_name")
                    or c.HasField("qualifier")
                    or c.timestamp_micros
                    or c.labels
                    or c.value
                ):
                    raise InvalidChunk("reset row with data")
                continue
            except CrossSync.StopIteration:
                raise InvalidChunk("premature end of stream")

    @staticmethod
    def _revise_request_rowset(
        row_set: RowSetPB,
        last_seen_row_key: bytes,
    ) -> RowSetPB:
        """
        Revise the rows in the request to avoid ones we've already processed.

        Args:
            row_set: the row set from the request
            last_seen_row_key: the last row key encountered
        Returns:
            RowSetPB: the new rowset after adusting for the last seen key
        Raises:
            _RowSetComplete: if there are no rows left to process after the revision
        """
        # if user is doing a whole table scan, start a new one with the last seen key
        if row_set is None or (not row_set.row_ranges and not row_set.row_keys):
            last_seen = last_seen_row_key
            return RowSetPB(row_ranges=[RowRangePB(start_key_open=last_seen)])
        # remove seen keys from user-specific key list
        adjusted_keys: list[bytes] = [
            k for k in row_set.row_keys if k > last_seen_row_key
        ]
        # adjust ranges to ignore keys before last seen
        adjusted_ranges: list[RowRangePB] = []
        for row_range in row_set.row_ranges:
            end_key = row_range.end_key_closed or row_range.end_key_open or None
            if end_key is None or end_key > last_seen_row_key:
                # end range is after last seen key
                new_range = RowRangePB(row_range)
                start_key = row_range.start_key_closed or row_range.start_key_open
                if start_key is None or start_key <= last_seen_row_key:
                    # replace start key with last seen
                    new_range.start_key_open = last_seen_row_key
                adjusted_ranges.append(new_range)
        if len(adjusted_keys) == 0 and len(adjusted_ranges) == 0:
            # if the query is empty after revision, raise an exception
            # this will avoid an unwanted full table scan
            raise _RowSetComplete()
        return RowSetPB(row_keys=adjusted_keys, row_ranges=adjusted_ranges)



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""The vertexai module."""

import importlib

from .client import Client

_evals = None


def __getattr__(name):  # type: ignore[no-untyped-def]
    if name == "evals":
        global _evals
        if _evals is None:
            try:
                _evals = importlib.import_module(".evals", __package__)
            except ImportError as e:
                raise ImportError(
                    "The 'evals' module requires additional dependencies. "
                    "Please install them using pip install "
                    "google-cloud-aiplatform[evaluation]"
                ) from e
        return _evals
    raise AttributeError(f"module '{__name__}' has no attribute '{name}'")


__all__ = [
    "Client",
    "evals",
]



# -*- coding: utf-8 -*-

# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""Schema for autorater metric configuration."""

AUTORATER_METRIC_SCHEMA = """
  $schema: https://json-schema.org/draft/2020-12/schema
  title: AutoRater Metric Configuration
  description: A metric definition for model-based evaluation.
  type: object
  properties:
    metadata:
      description: Useful information about the metric.
      type: object
      properties:
        name:
          description: Name of the metric.
          type: string
        description:
          description: Description of the metric.
          type: string
        author:
          description: Author of the metric.
          type: string
        contact:
          description: PoC for the metric.
          type: string
        version:
          description: Version of the metric.
          type: string
        classification:
          description: Classification of the metric.
          type: string
          enum:
            - experimental
            - benchmarked
            - deprecated
        required_inputs:
          description: Input fields used in the metric prompt template.
          type: array
          items:
            type: string
          minItems: 1
          uniqueItems: true
        benchmarks:
          description: List of benchmarks used for the metric.
          type: array
          items:
            type: object
            properties:
              dataset:
                description: Dataset used for benchmarking.
                type: string
              results:
                description: Results from benchmarking.
                type: string
            required:
              - results
          minItems: 1
          uniqueItems: true
        usage:
          description: Links to documentation or notebooks with example usage.
          type: array
          items:
            type: string
          minItems: 1
          uniqueItems: true
      required:
        - name
        - version
        - required_inputs
    steps:
      description: List of steps used for the autorater workflow.
      type: array
      items:
        type: object
        properties:
          type:
            description: Type of the step.
            type: string
            enum:
              - pointwise_metric
              - pairwise_metric
              - rubric
          prompt:
            description: Prompt template for the step.
            type: object
            properties:
              system_instruction:
                description: System instruction for the model.
                type: string
              template:
                description: Template to populate with inputs from the dataset.
                type: string
            required:
              - template
          model:
            description: Configuration of the model for the step.
            type: object
            properties:
              model_name_or_endpoint:
                description: Name or endpoint of the model.
                type: string
            required:
              - model_name_or_endpoint
          options:
            description: Options for the step.
            type: object
            properties:
              sample_count:
                description: Number of samples for each instance in the dataset.
                type: integer
              flip_enabled:
                description: Whether to flip candidate and baseline responses.
                type: boolean
          output:
            description: Output of the step.
            type: object
            properties:
              type:
                description: Type of the output.
                type: string
                enum:
                  - raw
            required:
              - type
        required:
          - type
          - prompt
      minItems: 1
      uniqueItems: true
  required:
    - metadata
    - steps
"""



#!/usr/bin/env python3
"""
Test Qdrant connection with the configured settings
"""

import os
from dotenv import load_dotenv

def test_qdrant_connection():
    """Test connection to Qdrant"""
    print("🧪 Testing Qdrant connection...")
    
    # Load environment variables
    load_dotenv()
    
    qdrant_url = os.getenv("QDRANT_URL", "http://localhost:6333")
    qdrant_api_key = os.getenv("QDRANT_API_KEY")
    
    print(f"📍 Qdrant URL: {qdrant_url}")
    
    try:
        from qdrant_client import QdrantClient
        print("✅ qdrant-client is available")
    except ImportError:
        print("❌ qdrant-client not available")
        print("Install with: pip install qdrant-client")
        return False
    
    try:
        # Create client
        client = QdrantClient(
            url=qdrant_url,
            api_key=qdrant_api_key,
            timeout=10
        )
        
        # Test connection by getting collections
        collections = client.get_collections()
        print(f"✅ Successfully connected to Qdrant")
        print(f"📊 Found {len(collections.collections)} collections")
        
        for collection in collections.collections:
            print(f"   - {collection.name}")
        
        # Test health
        try:
            health = client.get_cluster_info()
            print("✅ Cluster is healthy")
        except Exception as e:
            print(f"⚠️  Could not get cluster info: {e}")
        
        return True
        
    except Exception as e:
        print(f"❌ Failed to connect to Qdrant: {e}")
        print("Please check your QDRANT_URL and QDRANT_API_KEY in .env file")
        return False

if __name__ == "__main__":
    try:
        success = test_qdrant_connection()
        if success:
            print("\n🎉 Qdrant connection test passed!")
        else:
            print("\n❌ Qdrant connection test failed")
    except Exception as e:
        print(f"\n❌ Error during Qdrant test: {e}")
        import traceback
        traceback.print_exc()


# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import proto  # type: ignore

from google.cloud.bigtable_v2.types import data
from google.cloud.bigtable_v2.types import request_stats as gb_request_stats
from google.cloud.bigtable_v2.types import types
from google.protobuf import duration_pb2  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore
from google.protobuf import wrappers_pb2  # type: ignore
from google.rpc import status_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.bigtable.v2",
    manifest={
        "ReadRowsRequest",
        "ReadRowsResponse",
        "SampleRowKeysRequest",
        "SampleRowKeysResponse",
        "MutateRowRequest",
        "MutateRowResponse",
        "MutateRowsRequest",
        "MutateRowsResponse",
        "RateLimitInfo",
        "CheckAndMutateRowRequest",
        "CheckAndMutateRowResponse",
        "PingAndWarmRequest",
        "PingAndWarmResponse",
        "ReadModifyWriteRowRequest",
        "ReadModifyWriteRowResponse",
        "GenerateInitialChangeStreamPartitionsRequest",
        "GenerateInitialChangeStreamPartitionsResponse",
        "ReadChangeStreamRequest",
        "ReadChangeStreamResponse",
        "ExecuteQueryRequest",
        "ExecuteQueryResponse",
        "PrepareQueryRequest",
        "PrepareQueryResponse",
    },
)


class ReadRowsRequest(proto.Message):
    r"""Request message for Bigtable.ReadRows.

    Attributes:
        table_name (str):
            Optional. The unique name of the table from which to read.

            Values are of the form
            ``projects/<project>/instances/<instance>/tables/<table>``.
        authorized_view_name (str):
            Optional. The unique name of the AuthorizedView from which
            to read.

            Values are of the form
            ``projects/<project>/instances/<instance>/tables/<table>/authorizedViews/<authorized_view>``.
        materialized_view_name (str):
            Optional. The unique name of the MaterializedView from which
            to read.

            Values are of the form
            ``projects/<project>/instances/<instance>/materializedViews/<materialized_view>``.
        app_profile_id (str):
            This value specifies routing for replication.
            If not specified, the "default" application
            profile will be used.
        rows (google.cloud.bigtable_v2.types.RowSet):
            The row keys and/or ranges to read
            sequentially. If not specified, reads from all
            rows.
        filter (google.cloud.bigtable_v2.types.RowFilter):
            The filter to apply to the contents of the
            specified row(s). If unset, reads the entirety
            of each row.
        rows_limit (int):
            The read will stop after committing to N
            rows' worth of results. The default (zero) is to
            return all results.
        request_stats_view (google.cloud.bigtable_v2.types.ReadRowsRequest.RequestStatsView):
            The view into RequestStats, as described
            above.
        reversed (bool):
            Experimental API - Please note that this API is currently
            experimental and can change in the future.

            Return rows in lexiographical descending order of the row
            keys. The row contents will not be affected by this flag.

            Example result set:

            ::

                [
                  {key: "k2", "f:col1": "v1", "f:col2": "v1"},
                  {key: "k1", "f:col1": "v2", "f:col2": "v2"}
                ]
    """

    class RequestStatsView(proto.Enum):
        r"""The desired view into RequestStats that should be returned in
        the response.
        See also: RequestStats message.

        Values:
            REQUEST_STATS_VIEW_UNSPECIFIED (0):
                The default / unset value. The API will
                default to the NONE option below.
            REQUEST_STATS_NONE (1):
                Do not include any RequestStats in the
                response. This will leave the RequestStats
                embedded message unset in the response.
            REQUEST_STATS_FULL (2):
                Include the full set of available
                RequestStats in the response, applicable to this
                read.
        """
        REQUEST_STATS_VIEW_UNSPECIFIED = 0
        REQUEST_STATS_NONE = 1
        REQUEST_STATS_FULL = 2

    table_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    authorized_view_name: str = proto.Field(
        proto.STRING,
        number=9,
    )
    materialized_view_name: str = proto.Field(
        proto.STRING,
        number=11,
    )
    app_profile_id: str = proto.Field(
        proto.STRING,
        number=5,
    )
    rows: data.RowSet = proto.Field(
        proto.MESSAGE,
        number=2,
        message=data.RowSet,
    )
    filter: data.RowFilter = proto.Field(
        proto.MESSAGE,
        number=3,
        message=data.RowFilter,
    )
    rows_limit: int = proto.Field(
        proto.INT64,
        number=4,
    )
    request_stats_view: RequestStatsView = proto.Field(
        proto.ENUM,
        number=6,
        enum=RequestStatsView,
    )
    reversed: bool = proto.Field(
        proto.BOOL,
        number=7,
    )


class ReadRowsResponse(proto.Message):
    r"""Response message for Bigtable.ReadRows.

    Attributes:
        chunks (MutableSequence[google.cloud.bigtable_v2.types.ReadRowsResponse.CellChunk]):
            A collection of a row's contents as part of
            the read request.
        last_scanned_row_key (bytes):
            Optionally the server might return the row
            key of the last row it has scanned.  The client
            can use this to construct a more efficient retry
            request if needed: any row keys or portions of
            ranges less than this row key can be dropped
            from the request. This is primarily useful for
            cases where the server has read a lot of data
            that was filtered out since the last committed
            row key, allowing the client to skip that work
            on a retry.
        request_stats (google.cloud.bigtable_v2.types.RequestStats):
            If requested, return enhanced query performance statistics.
            The field request_stats is empty in a streamed response
            unless the ReadRowsResponse message contains request_stats
            in the last message of the stream. Always returned when
            requested, even when the read request returns an empty
            response.
    """

    class CellChunk(proto.Message):
        r"""Specifies a piece of a row's contents returned as part of the
        read response stream.

        This message has `oneof`_ fields (mutually exclusive fields).
        For each oneof, at most one member field can be set at the same time.
        Setting any member of the oneof automatically clears all other
        members.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            row_key (bytes):
                The row key for this chunk of data.  If the
                row key is empty, this CellChunk is a
                continuation of the same row as the previous
                CellChunk in the response stream, even if that
                CellChunk was in a previous ReadRowsResponse
                message.
            family_name (google.protobuf.wrappers_pb2.StringValue):
                The column family name for this chunk of data. If this
                message is not present this CellChunk is a continuation of
                the same column family as the previous CellChunk. The empty
                string can occur as a column family name in a response so
                clients must check explicitly for the presence of this
                message, not just for ``family_name.value`` being non-empty.
            qualifier (google.protobuf.wrappers_pb2.BytesValue):
                The column qualifier for this chunk of data. If this message
                is not present, this CellChunk is a continuation of the same
                column as the previous CellChunk. Column qualifiers may be
                empty so clients must check for the presence of this
                message, not just for ``qualifier.value`` being non-empty.
            timestamp_micros (int):
                The cell's stored timestamp, which also uniquely identifies
                it within its column. Values are always expressed in
                microseconds, but individual tables may set a coarser
                granularity to further restrict the allowed values. For
                example, a table which specifies millisecond granularity
                will only allow values of ``timestamp_micros`` which are
                multiples of 1000. Timestamps are only set in the first
                CellChunk per cell (for cells split into multiple chunks).
            labels (MutableSequence[str]):
                Labels applied to the cell by a
                [RowFilter][google.bigtable.v2.RowFilter]. Labels are only
                set on the first CellChunk per cell.
            value (bytes):
                The value stored in the cell.  Cell values
                can be split across multiple CellChunks.  In
                that case only the value field will be set in
                CellChunks after the first: the timestamp and
                labels will only be present in the first
                CellChunk, even if the first CellChunk came in a
                previous ReadRowsResponse.
            value_size (int):
                If this CellChunk is part of a chunked cell value and this
                is not the final chunk of that cell, value_size will be set
                to the total length of the cell value. The client can use
                this size to pre-allocate memory to hold the full cell
                value.
            reset_row (bool):
                Indicates that the client should drop all previous chunks
                for ``row_key``, as it will be re-read from the beginning.

                This field is a member of `oneof`_ ``row_status``.
            commit_row (bool):
                Indicates that the client can safely process all previous
                chunks for ``row_key``, as its data has been fully read.

                This field is a member of `oneof`_ ``row_status``.
        """

        row_key: bytes = proto.Field(
            proto.BYTES,
            number=1,
        )
        family_name: wrappers_pb2.StringValue = proto.Field(
            proto.MESSAGE,
            number=2,
            message=wrappers_pb2.StringValue,
        )
        qualifier: wrappers_pb2.BytesValue = proto.Field(
            proto.MESSAGE,
            number=3,
            message=wrappers_pb2.BytesValue,
        )
        timestamp_micros: int = proto.Field(
            proto.INT64,
            number=4,
        )
        labels: MutableSequence[str] = proto.RepeatedField(
            proto.STRING,
            number=5,
        )
        value: bytes = proto.Field(
            proto.BYTES,
            number=6,
        )
        value_size: int = proto.Field(
            proto.INT32,
            number=7,
        )
        reset_row: bool = proto.Field(
            proto.BOOL,
            number=8,
            oneof="row_status",
        )
        commit_row: bool = proto.Field(
            proto.BOOL,
            number=9,
            oneof="row_status",
        )

    chunks: MutableSequence[CellChunk] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=CellChunk,
    )
    last_scanned_row_key: bytes = proto.Field(
        proto.BYTES,
        number=2,
    )
    request_stats: gb_request_stats.RequestStats = proto.Field(
        proto.MESSAGE,
        number=3,
        message=gb_request_stats.RequestStats,
    )


class SampleRowKeysRequest(proto.Message):
    r"""Request message for Bigtable.SampleRowKeys.

    Attributes:
        table_name (str):
            Optional. The unique name of the table from which to sample
            row keys.

            Values are of the form
            ``projects/<project>/instances/<instance>/tables/<table>``.
        authorized_view_name (str):
            Optional. The unique name of the AuthorizedView from which
            to sample row keys.

            Values are of the form
            ``projects/<project>/instances/<instance>/tables/<table>/authorizedViews/<authorized_view>``.
        materialized_view_name (str):
            Optional. The unique name of the MaterializedView from which
            to read.

            Values are of the form
            ``projects/<project>/instances/<instance>/materializedViews/<materialized_view>``.
        app_profile_id (str):
            This value specifies routing for replication.
            If not specified, the "default" application
            profile will be used.
    """

    table_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    authorized_view_name: str = proto.Field(
        proto.STRING,
        number=4,
    )
    materialized_view_name: str = proto.Field(
        proto.STRING,
        number=5,
    )
    app_profile_id: str = proto.Field(
        proto.STRING,
        number=2,
    )


class SampleRowKeysResponse(proto.Message):
    r"""Response message for Bigtable.SampleRowKeys.

    Attributes:
        row_key (bytes):
            Sorted streamed sequence of sample row keys
            in the table. The table might have contents
            before the first row key in the list and after
            the last one, but a key containing the empty
            string indicates "end of table" and will be the
            last response given, if present.
            Note that row keys in this list may not have
            ever been written to or read from, and users
            should therefore not make any assumptions about
            the row key structure that are specific to their
            use case.
        offset_bytes (int):
            Approximate total storage space used by all rows in the
            table which precede ``row_key``. Buffering the contents of
            all rows between two subsequent samples would require space
            roughly equal to the difference in their ``offset_bytes``
            fields.
    """

    row_key: bytes = proto.Field(
        proto.BYTES,
        number=1,
    )
    offset_bytes: int = proto.Field(
        proto.INT64,
        number=2,
    )


class MutateRowRequest(proto.Message):
    r"""Request message for Bigtable.MutateRow.

    Attributes:
        table_name (str):
            Optional. The unique name of the table to which the mutation
            should be applied.

            Values are of the form
            ``projects/<project>/instances/<instance>/tables/<table>``.
        authorized_view_name (str):
            Optional. The unique name of the AuthorizedView to which the
            mutation should be applied.

            Values are of the form
            ``projects/<project>/instances/<instance>/tables/<table>/authorizedViews/<authorized_view>``.
        app_profile_id (str):
            This value specifies routing for replication.
            If not specified, the "default" application
            profile will be used.
        row_key (bytes):
            Required. The key of the row to which the
            mutation should be applied.
        mutations (MutableSequence[google.cloud.bigtable_v2.types.Mutation]):
            Required. Changes to be atomically applied to
            the specified row. Entries are applied in order,
            meaning that earlier mutations can be masked by
            later ones. Must contain at least one entry and
            at most 100000.
        idempotency (google.cloud.bigtable_v2.types.Idempotency):
            If set consistently across retries, prevents
            this mutation from being double applied to
            aggregate column families within a 15m window.
    """

    table_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    authorized_view_name: str = proto.Field(
        proto.STRING,
        number=6,
    )
    app_profile_id: str = proto.Field(
        proto.STRING,
        number=4,
    )
    row_key: bytes = proto.Field(
        proto.BYTES,
        number=2,
    )
    mutations: MutableSequence[data.Mutation] = proto.RepeatedField(
        proto.MESSAGE,
        number=3,
        message=data.Mutation,
    )
    idempotency: data.Idempotency = proto.Field(
        proto.MESSAGE,
        number=8,
        message=data.Idempotency,
    )


class MutateRowResponse(proto.Message):
    r"""Response message for Bigtable.MutateRow."""


class MutateRowsRequest(proto.Message):
    r"""Request message for BigtableService.MutateRows.

    Attributes:
        table_name (str):
            Optional. The unique name of the table to which the
            mutations should be applied.

            Values are of the form
            ``projects/<project>/instances/<instance>/tables/<table>``.
        authorized_view_name (str):
            Optional. The unique name of the AuthorizedView to which the
            mutations should be applied.

            Values are of the form
            ``projects/<project>/instances/<instance>/tables/<table>/authorizedViews/<authorized_view>``.
        app_profile_id (str):
            This value specifies routing for replication.
            If not specified, the "default" application
            profile will be used.
        entries (MutableSequence[google.cloud.bigtable_v2.types.MutateRowsRequest.Entry]):
            Required. The row keys and corresponding
            mutations to be applied in bulk. Each entry is
            applied as an atomic mutation, but the entries
            may be applied in arbitrary order (even between
            entries for the same row). At least one entry
            must be specified, and in total the entries can
            contain at most 100000 mutations.
    """

    class Entry(proto.Message):
        r"""A mutation for a given row.

        Attributes:
            row_key (bytes):
                The key of the row to which the ``mutations`` should be
                applied.
            mutations (MutableSequence[google.cloud.bigtable_v2.types.Mutation]):
                Required. Changes to be atomically applied to
                the specified row. Mutations are applied in
                order, meaning that earlier mutations can be
                masked by later ones. You must specify at least
                one mutation.
            idempotency (google.cloud.bigtable_v2.types.Idempotency):
                If set consistently across retries, prevents
                this mutation from being double applied to
                aggregate column families within a 15m window.
        """

        row_key: bytes = proto.Field(
            proto.BYTES,
            number=1,
        )
        mutations: MutableSequence[data.Mutation] = proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message=data.Mutation,
        )
        idempotency: data.Idempotency = proto.Field(
            proto.MESSAGE,
            number=3,
            message=data.Idempotency,
        )

    table_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    authorized_view_name: str = proto.Field(
        proto.STRING,
        number=5,
    )
    app_profile_id: str = proto.Field(
        proto.STRING,
        number=3,
    )
    entries: MutableSequence[Entry] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message=Entry,
    )


class MutateRowsResponse(proto.Message):
    r"""Response message for BigtableService.MutateRows.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        entries (MutableSequence[google.cloud.bigtable_v2.types.MutateRowsResponse.Entry]):
            One or more results for Entries from the
            batch request.
        rate_limit_info (google.cloud.bigtable_v2.types.RateLimitInfo):
            Information about how client should limit the
            rate (QPS). Primirily used by supported official
            Cloud Bigtable clients. If unset, the rate limit
            info is not provided by the server.

            This field is a member of `oneof`_ ``_rate_limit_info``.
    """

    class Entry(proto.Message):
        r"""The result of applying a passed mutation in the original
        request.

        Attributes:
            index (int):
                The index into the original request's ``entries`` list of
                the Entry for which a result is being reported.
            status (google.rpc.status_pb2.Status):
                The result of the request Entry identified by ``index``.
                Depending on how requests are batched during execution, it
                is possible for one Entry to fail due to an error with
                another Entry. In the event that this occurs, the same error
                will be reported for both entries.
        """

        index: int = proto.Field(
            proto.INT64,
            number=1,
        )
        status: status_pb2.Status = proto.Field(
            proto.MESSAGE,
            number=2,
            message=status_pb2.Status,
        )

    entries: MutableSequence[Entry] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=Entry,
    )
    rate_limit_info: "RateLimitInfo" = proto.Field(
        proto.MESSAGE,
        number=3,
        optional=True,
        message="RateLimitInfo",
    )


class RateLimitInfo(proto.Message):
    r"""Information about how client should adjust the load to
    Bigtable.

    Attributes:
        period (google.protobuf.duration_pb2.Duration):
            Time that clients should wait before
            adjusting the target rate again. If clients
            adjust rate too frequently, the impact of the
            previous adjustment may not have been taken into
            account and may over-throttle or under-throttle.
            If clients adjust rate too slowly, they will not
            be responsive to load changes on server side,
            and may over-throttle or under-throttle.
        factor (float):
            If it has been at least one ``period`` since the last load
            adjustment, the client should multiply the current load by
            this value to get the new target load. For example, if the
            current load is 100 and ``factor`` is 0.8, the new target
            load should be 80. After adjusting, the client should ignore
            ``factor`` until another ``period`` has passed.

            The client can measure its load using any unit that's
            comparable over time. For example, QPS can be used as long
            as each request involves a similar amount of work.
    """

    period: duration_pb2.Duration = proto.Field(
        proto.MESSAGE,
        number=1,
        message=duration_pb2.Duration,
    )
    factor: float = proto.Field(
        proto.DOUBLE,
        number=2,
    )


class CheckAndMutateRowRequest(proto.Message):
    r"""Request message for Bigtable.CheckAndMutateRow.

    Attributes:
        table_name (str):
            Optional. The unique name of the table to which the
            conditional mutation should be applied.

            Values are of the form
            ``projects/<project>/instances/<instance>/tables/<table>``.
        authorized_view_name (str):
            Optional. The unique name of the AuthorizedView to which the
            conditional mutation should be applied.

            Values are of the form
            ``projects/<project>/instances/<instance>/tables/<table>/authorizedViews/<authorized_view>``.
        app_profile_id (str):
            This value specifies routing for replication.
            If not specified, the "default" application
            profile will be used.
        row_key (bytes):
            Required. The key of the row to which the
            conditional mutation should be applied.
        predicate_filter (google.cloud.bigtable_v2.types.RowFilter):
            The filter to be applied to the contents of the specified
            row. Depending on whether or not any results are yielded,
            either ``true_mutations`` or ``false_mutations`` will be
            executed. If unset, checks that the row contains any values
            at all.
        true_mutations (MutableSequence[google.cloud.bigtable_v2.types.Mutation]):
            Changes to be atomically applied to the specified row if
            ``predicate_filter`` yields at least one cell when applied
            to ``row_key``. Entries are applied in order, meaning that
            earlier mutations can be masked by later ones. Must contain
            at least one entry if ``false_mutations`` is empty, and at
            most 100000.
        false_mutations (MutableSequence[google.cloud.bigtable_v2.types.Mutation]):
            Changes to be atomically applied to the specified row if
            ``predicate_filter`` does not yield any cells when applied
            to ``row_key``. Entries are applied in order, meaning that
            earlier mutations can be masked by later ones. Must contain
            at least one entry if ``true_mutations`` is empty, and at
            most 100000.
    """

    table_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    authorized_view_name: str = proto.Field(
        proto.STRING,
        number=9,
    )
    app_profile_id: str = proto.Field(
        proto.STRING,
        number=7,
    )
    row_key: bytes = proto.Field(
        proto.BYTES,
        number=2,
    )
    predicate_filter: data.RowFilter = proto.Field(
        proto.MESSAGE,
        number=6,
        message=data.RowFilter,
    )
    true_mutations: MutableSequence[data.Mutation] = proto.RepeatedField(
        proto.MESSAGE,
        number=4,
        message=data.Mutation,
    )
    false_mutations: MutableSequence[data.Mutation] = proto.RepeatedField(
        proto.MESSAGE,
        number=5,
        message=data.Mutation,
    )


class CheckAndMutateRowResponse(proto.Message):
    r"""Response message for Bigtable.CheckAndMutateRow.

    Attributes:
        predicate_matched (bool):
            Whether or not the request's ``predicate_filter`` yielded
            any results for the specified row.
    """

    predicate_matched: bool = proto.Field(
        proto.BOOL,
        number=1,
    )


class PingAndWarmRequest(proto.Message):
    r"""Request message for client connection keep-alive and warming.

    Attributes:
        name (str):
            Required. The unique name of the instance to check
            permissions for as well as respond. Values are of the form
            ``projects/<project>/instances/<instance>``.
        app_profile_id (str):
            This value specifies routing for replication.
            If not specified, the "default" application
            profile will be used.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    app_profile_id: str = proto.Field(
        proto.STRING,
        number=2,
    )


class PingAndWarmResponse(proto.Message):
    r"""Response message for Bigtable.PingAndWarm connection
    keepalive and warming.

    """


class ReadModifyWriteRowRequest(proto.Message):
    r"""Request message for Bigtable.ReadModifyWriteRow.

    Attributes:
        table_name (str):
            Optional. The unique name of the table to which the
            read/modify/write rules should be applied.

            Values are of the form
            ``projects/<project>/instances/<instance>/tables/<table>``.
        authorized_view_name (str):
            Optional. The unique name of the AuthorizedView to which the
            read/modify/write rules should be applied.

            Values are of the form
            ``projects/<project>/instances/<instance>/tables/<table>/authorizedViews/<authorized_view>``.
        app_profile_id (str):
            This value specifies routing for replication.
            If not specified, the "default" application
            profile will be used.
        row_key (bytes):
            Required. The key of the row to which the
            read/modify/write rules should be applied.
        rules (MutableSequence[google.cloud.bigtable_v2.types.ReadModifyWriteRule]):
            Required. Rules specifying how the specified
            row's contents are to be transformed into
            writes. Entries are applied in order, meaning
            that earlier rules will affect the results of
            later ones. At least one entry must be
            specified, and there can be at most 100000
            rules.
    """

    table_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    authorized_view_name: str = proto.Field(
        proto.STRING,
        number=6,
    )
    app_profile_id: str = proto.Field(
        proto.STRING,
        number=4,
    )
    row_key: bytes = proto.Field(
        proto.BYTES,
        number=2,
    )
    rules: MutableSequence[data.ReadModifyWriteRule] = proto.RepeatedField(
        proto.MESSAGE,
        number=3,
        message=data.ReadModifyWriteRule,
    )


class ReadModifyWriteRowResponse(proto.Message):
    r"""Response message for Bigtable.ReadModifyWriteRow.

    Attributes:
        row (google.cloud.bigtable_v2.types.Row):
            A Row containing the new contents of all
            cells modified by the request.
    """

    row: data.Row = proto.Field(
        proto.MESSAGE,
        number=1,
        message=data.Row,
    )


class GenerateInitialChangeStreamPartitionsRequest(proto.Message):
    r"""NOTE: This API is intended to be used by Apache Beam
    BigtableIO. Request message for
    Bigtable.GenerateInitialChangeStreamPartitions.

    Attributes:
        table_name (str):
            Required. The unique name of the table from which to get
            change stream partitions. Values are of the form
            ``projects/<project>/instances/<instance>/tables/<table>``.
            Change streaming must be enabled on the table.
        app_profile_id (str):
            This value specifies routing for replication.
            If not specified, the "default" application
            profile will be used. Single cluster routing
            must be configured on the profile.
    """

    table_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    app_profile_id: str = proto.Field(
        proto.STRING,
        number=2,
    )


class GenerateInitialChangeStreamPartitionsResponse(proto.Message):
    r"""NOTE: This API is intended to be used by Apache Beam
    BigtableIO. Response message for
    Bigtable.GenerateInitialChangeStreamPartitions.

    Attributes:
        partition (google.cloud.bigtable_v2.types.StreamPartition):
            A partition of the change stream.
    """

    partition: data.StreamPartition = proto.Field(
        proto.MESSAGE,
        number=1,
        message=data.StreamPartition,
    )


class ReadChangeStreamRequest(proto.Message):
    r"""NOTE: This API is intended to be used by Apache Beam
    BigtableIO. Request message for Bigtable.ReadChangeStream.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        table_name (str):
            Required. The unique name of the table from which to read a
            change stream. Values are of the form
            ``projects/<project>/instances/<instance>/tables/<table>``.
            Change streaming must be enabled on the table.
        app_profile_id (str):
            This value specifies routing for replication.
            If not specified, the "default" application
            profile will be used. Single cluster routing
            must be configured on the profile.
        partition (google.cloud.bigtable_v2.types.StreamPartition):
            The partition to read changes from.
        start_time (google.protobuf.timestamp_pb2.Timestamp):
            Start reading the stream at the specified
            timestamp. This timestamp must be within the
            change stream retention period, less than or
            equal to the current time, and after change
            stream creation, whichever is greater. This
            value is inclusive and will be truncated to
            microsecond granularity.

            This field is a member of `oneof`_ ``start_from``.
        continuation_tokens (google.cloud.bigtable_v2.types.StreamContinuationTokens):
            Tokens that describe how to resume reading a stream where
            reading previously left off. If specified, changes will be
            read starting at the the position. Tokens are delivered on
            the stream as part of ``Heartbeat`` and ``CloseStream``
            messages.

            If a single token is provided, the token's partition must
            exactly match the request's partition. If multiple tokens
            are provided, as in the case of a partition merge, the union
            of the token partitions must exactly cover the request's
            partition. Otherwise, INVALID_ARGUMENT will be returned.

            This field is a member of `oneof`_ ``start_from``.
        end_time (google.protobuf.timestamp_pb2.Timestamp):
            If specified, OK will be returned when the
            stream advances beyond this time. Otherwise,
            changes will be continuously delivered on the
            stream. This value is inclusive and will be
            truncated to microsecond granularity.
        heartbeat_duration (google.protobuf.duration_pb2.Duration):
            If specified, the duration between ``Heartbeat`` messages on
            the stream. Otherwise, defaults to 5 seconds.
    """

    table_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    app_profile_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    partition: data.StreamPartition = proto.Field(
        proto.MESSAGE,
        number=3,
        message=data.StreamPartition,
    )
    start_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="start_from",
        message=timestamp_pb2.Timestamp,
    )
    continuation_tokens: data.StreamContinuationTokens = proto.Field(
        proto.MESSAGE,
        number=6,
        oneof="start_from",
        message=data.StreamContinuationTokens,
    )
    end_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=5,
        message=timestamp_pb2.Timestamp,
    )
    heartbeat_duration: duration_pb2.Duration = proto.Field(
        proto.MESSAGE,
        number=7,
        message=duration_pb2.Duration,
    )


class ReadChangeStreamResponse(proto.Message):
    r"""NOTE: This API is intended to be used by Apache Beam
    BigtableIO. Response message for Bigtable.ReadChangeStream.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        data_change (google.cloud.bigtable_v2.types.ReadChangeStreamResponse.DataChange):
            A mutation to the partition.

            This field is a member of `oneof`_ ``stream_record``.
        heartbeat (google.cloud.bigtable_v2.types.ReadChangeStreamResponse.Heartbeat):
            A periodic heartbeat message.

            This field is a member of `oneof`_ ``stream_record``.
        close_stream (google.cloud.bigtable_v2.types.ReadChangeStreamResponse.CloseStream):
            An indication that the stream should be
            closed.

            This field is a member of `oneof`_ ``stream_record``.
    """

    class MutationChunk(proto.Message):
        r"""A partial or complete mutation.

        Attributes:
            chunk_info (google.cloud.bigtable_v2.types.ReadChangeStreamResponse.MutationChunk.ChunkInfo):
                If set, then the mutation is a ``SetCell`` with a chunked
                value across multiple messages.
            mutation (google.cloud.bigtable_v2.types.Mutation):
                If this is a continuation of a chunked message
                (``chunked_value_offset`` > 0), ignore all fields except the
                ``SetCell``'s value and merge it with the previous message
                by concatenating the value fields.
        """

        class ChunkInfo(proto.Message):
            r"""Information about the chunking of this mutation. Only ``SetCell``
            mutations can be chunked, and all chunks for a ``SetCell`` will be
            delivered contiguously with no other mutation types interleaved.

            Attributes:
                chunked_value_size (int):
                    The total value size of all the chunks that make up the
                    ``SetCell``.
                chunked_value_offset (int):
                    The byte offset of this chunk into the total
                    value size of the mutation.
                last_chunk (bool):
                    When true, this is the last chunk of a chunked ``SetCell``.
            """

            chunked_value_size: int = proto.Field(
                proto.INT32,
                number=1,
            )
            chunked_value_offset: int = proto.Field(
                proto.INT32,
                number=2,
            )
            last_chunk: bool = proto.Field(
                proto.BOOL,
                number=3,
            )

        chunk_info: "ReadChangeStreamResponse.MutationChunk.ChunkInfo" = proto.Field(
            proto.MESSAGE,
            number=1,
            message="ReadChangeStreamResponse.MutationChunk.ChunkInfo",
        )
        mutation: data.Mutation = proto.Field(
            proto.MESSAGE,
            number=2,
            message=data.Mutation,
        )

    class DataChange(proto.Message):
        r"""A message corresponding to one or more mutations to the partition
        being streamed. A single logical ``DataChange`` message may also be
        split across a sequence of multiple individual messages. Messages
        other than the first in a sequence will only have the ``type`` and
        ``chunks`` fields populated, with the final message in the sequence
        also containing ``done`` set to true.

        Attributes:
            type_ (google.cloud.bigtable_v2.types.ReadChangeStreamResponse.DataChange.Type):
                The type of the mutation.
            source_cluster_id (str):
                The cluster where the mutation was applied. Not set when
                ``type`` is ``GARBAGE_COLLECTION``.
            row_key (bytes):
                The row key for all mutations that are part of this
                ``DataChange``. If the ``DataChange`` is chunked across
                multiple messages, then this field will only be set for the
                first message.
            commit_timestamp (google.protobuf.timestamp_pb2.Timestamp):
                The timestamp at which the mutation was
                applied on the Bigtable server.
            tiebreaker (int):
                A value that lets stream consumers reconstruct Bigtable's
                conflict resolution semantics.
                https://cloud.google.com/bigtable/docs/writes#conflict-resolution
                In the event that the same row key, column family, column
                qualifier, timestamp are modified on different clusters at
                the same ``commit_timestamp``, the mutation with the larger
                ``tiebreaker`` will be the one chosen for the eventually
                consistent state of the system.
            chunks (MutableSequence[google.cloud.bigtable_v2.types.ReadChangeStreamResponse.MutationChunk]):
                The mutations associated with this change to the partition.
                May contain complete mutations or chunks of a multi-message
                chunked ``DataChange`` record.
            done (bool):
                When true, indicates that the entire ``DataChange`` has been
                read and the client can safely process the message.
            token (str):
                An encoded position for this stream's
                partition to restart reading from. This token is
                for the StreamPartition from the request.
            estimated_low_watermark (google.protobuf.timestamp_pb2.Timestamp):
                An estimate of the commit timestamp that is
                usually lower than or equal to any timestamp for
                a record that will be delivered in the future on
                the stream. It is possible that, under
                particular circumstances that a future record
                has a timestamp that is lower than a previously
                seen timestamp. For an example usage see
                https://beam.apache.org/documentation/basics/#watermarks
        """

        class Type(proto.Enum):
            r"""The type of mutation.

            Values:
                TYPE_UNSPECIFIED (0):
                    The type is unspecified.
                USER (1):
                    A user-initiated mutation.
                GARBAGE_COLLECTION (2):
                    A system-initiated mutation as part of
                    garbage collection.
                    https://cloud.google.com/bigtable/docs/garbage-collection
                CONTINUATION (3):
                    This is a continuation of a multi-message
                    change.
            """
            TYPE_UNSPECIFIED = 0
            USER = 1
            GARBAGE_COLLECTION = 2
            CONTINUATION = 3

        type_: "ReadChangeStreamResponse.DataChange.Type" = proto.Field(
            proto.ENUM,
            number=1,
            enum="ReadChangeStreamResponse.DataChange.Type",
        )
        source_cluster_id: str = proto.Field(
            proto.STRING,
            number=2,
        )
        row_key: bytes = proto.Field(
            proto.BYTES,
            number=3,
        )
        commit_timestamp: timestamp_pb2.Timestamp = proto.Field(
            proto.MESSAGE,
            number=4,
            message=timestamp_pb2.Timestamp,
        )
        tiebreaker: int = proto.Field(
            proto.INT32,
            number=5,
        )
        chunks: MutableSequence[
            "ReadChangeStreamResponse.MutationChunk"
        ] = proto.RepeatedField(
            proto.MESSAGE,
            number=6,
            message="ReadChangeStreamResponse.MutationChunk",
        )
        done: bool = proto.Field(
            proto.BOOL,
            number=8,
        )
        token: str = proto.Field(
            proto.STRING,
            number=9,
        )
        estimated_low_watermark: timestamp_pb2.Timestamp = proto.Field(
            proto.MESSAGE,
            number=10,
            message=timestamp_pb2.Timestamp,
        )

    class Heartbeat(proto.Message):
        r"""A periodic message with information that can be used to
        checkpoint the state of a stream.

        Attributes:
            continuation_token (google.cloud.bigtable_v2.types.StreamContinuationToken):
                A token that can be provided to a subsequent
                ``ReadChangeStream`` call to pick up reading at the current
                stream position.
            estimated_low_watermark (google.protobuf.timestamp_pb2.Timestamp):
                An estimate of the commit timestamp that is
                usually lower than or equal to any timestamp for
                a record that will be delivered in the future on
                the stream. It is possible that, under
                particular circumstances that a future record
                has a timestamp that is lower than a previously
                seen timestamp. For an example usage see
                https://beam.apache.org/documentation/basics/#watermarks
        """

        continuation_token: data.StreamContinuationToken = proto.Field(
            proto.MESSAGE,
            number=1,
            message=data.StreamContinuationToken,
        )
        estimated_low_watermark: timestamp_pb2.Timestamp = proto.Field(
            proto.MESSAGE,
            number=2,
            message=timestamp_pb2.Timestamp,
        )

    class CloseStream(proto.Message):
        r"""A message indicating that the client should stop reading from the
        stream. If status is OK and ``continuation_tokens`` &
        ``new_partitions`` are empty, the stream has finished (for example
        if there was an ``end_time`` specified). If ``continuation_tokens``
        & ``new_partitions`` are present, then a change in partitioning
        requires the client to open a new stream for each token to resume
        reading. Example:

        ::

                                             [B,      D) ends
                                                  |
                                                  v
                          new_partitions:  [A,  C) [C,  E)
            continuation_tokens.partitions:  [B,C) [C,D)
                                             ^---^ ^---^
                                             ^     ^
                                             |     |
                                             |     StreamContinuationToken 2
                                             |
                                             StreamContinuationToken 1

        To read the new partition [A,C), supply the continuation tokens
        whose ranges cover the new partition, for example
        ContinuationToken[A,B) & ContinuationToken[B,C).

        Attributes:
            status (google.rpc.status_pb2.Status):
                The status of the stream.
            continuation_tokens (MutableSequence[google.cloud.bigtable_v2.types.StreamContinuationToken]):
                If non-empty, contains the information needed
                to resume reading their associated partitions.
            new_partitions (MutableSequence[google.cloud.bigtable_v2.types.StreamPartition]):
                If non-empty, contains the new partitions to start reading
                from, which are related to but not necessarily identical to
                the partitions for the above ``continuation_tokens``.
        """

        status: status_pb2.Status = proto.Field(
            proto.MESSAGE,
            number=1,
            message=status_pb2.Status,
        )
        continuation_tokens: MutableSequence[
            data.StreamContinuationToken
        ] = proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message=data.StreamContinuationToken,
        )
        new_partitions: MutableSequence[data.StreamPartition] = proto.RepeatedField(
            proto.MESSAGE,
            number=3,
            message=data.StreamPartition,
        )

    data_change: DataChange = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="stream_record",
        message=DataChange,
    )
    heartbeat: Heartbeat = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="stream_record",
        message=Heartbeat,
    )
    close_stream: CloseStream = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof="stream_record",
        message=CloseStream,
    )


class ExecuteQueryRequest(proto.Message):
    r"""Request message for Bigtable.ExecuteQuery

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        instance_name (str):
            Required. The unique name of the instance against which the
            query should be executed. Values are of the form
            ``projects/<project>/instances/<instance>``
        app_profile_id (str):
            Optional. This value specifies routing for replication. If
            not specified, the ``default`` application profile will be
            used.
        query (str):
            Required. The query string.

            Exactly one of ``query`` and ``prepared_query`` is required.
            Setting both or neither is an ``INVALID_ARGUMENT``.
        prepared_query (bytes):
            A prepared query that was returned from
            ``PrepareQueryResponse``.

            Exactly one of ``query`` and ``prepared_query`` is required.
            Setting both or neither is an ``INVALID_ARGUMENT``.

            Setting this field also places restrictions on several other
            fields:

            - ``data_format`` must be empty.
            - ``validate_only`` must be false.
            - ``params`` must match the ``param_types`` set in the
              ``PrepareQueryRequest``.
        proto_format (google.cloud.bigtable_v2.types.ProtoFormat):
            Protocol buffer format as described by
            ProtoSchema and ProtoRows messages.

            This field is a member of `oneof`_ ``data_format``.
        resume_token (bytes):
            Optional. If this request is resuming a previously
            interrupted query execution, ``resume_token`` should be
            copied from the last PartialResultSet yielded before the
            interruption. Doing this enables the query execution to
            resume where the last one left off. The rest of the request
            parameters must exactly match the request that yielded this
            token. Otherwise the request will fail.
        params (MutableMapping[str, google.cloud.bigtable_v2.types.Value]):
            Required. params contains string type keys and Bigtable type
            values that bind to placeholders in the query string. In
            query string, a parameter placeholder consists of the ``@``
            character followed by the parameter name (for example,
            ``@firstName``) in the query string.

            For example, if
            ``params["firstName"] = bytes_value: "foo" type {bytes_type {}}``
            then ``@firstName`` will be replaced with googlesql bytes
            value "foo" in the query string during query evaluation.

            If ``Value.kind`` is not set, the value is treated as a NULL
            value of the given type. For example, if
            ``params["firstName"] = type {string_type {}}`` then
            ``@firstName`` will be replaced with googlesql null string.

            If ``query`` is set, any empty ``Value.type`` in the map
            will be rejected with ``INVALID_ARGUMENT``.

            If ``prepared_query`` is set, any empty ``Value.type`` in
            the map will be inferred from the ``param_types`` in the
            ``PrepareQueryRequest``. Any non-empty ``Value.type`` must
            match the corresponding ``param_types`` entry, or be
            rejected with ``INVALID_ARGUMENT``.
    """

    instance_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    app_profile_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    query: str = proto.Field(
        proto.STRING,
        number=3,
    )
    prepared_query: bytes = proto.Field(
        proto.BYTES,
        number=9,
    )
    proto_format: data.ProtoFormat = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="data_format",
        message=data.ProtoFormat,
    )
    resume_token: bytes = proto.Field(
        proto.BYTES,
        number=8,
    )
    params: MutableMapping[str, data.Value] = proto.MapField(
        proto.STRING,
        proto.MESSAGE,
        number=7,
        message=data.Value,
    )


class ExecuteQueryResponse(proto.Message):
    r"""Response message for Bigtable.ExecuteQuery

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        metadata (google.cloud.bigtable_v2.types.ResultSetMetadata):
            Structure of rows in this response stream.
            The first (and only the first) response streamed
            from the server will be of this type.

            This field is a member of `oneof`_ ``response``.
        results (google.cloud.bigtable_v2.types.PartialResultSet):
            A partial result set with row data
            potentially including additional instructions on
            how recent past and future partial responses
            should be interpreted.

            This field is a member of `oneof`_ ``response``.
    """

    metadata: data.ResultSetMetadata = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="response",
        message=data.ResultSetMetadata,
    )
    results: data.PartialResultSet = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="response",
        message=data.PartialResultSet,
    )


class PrepareQueryRequest(proto.Message):
    r"""Request message for Bigtable.PrepareQuery

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        instance_name (str):
            Required. The unique name of the instance against which the
            query should be executed. Values are of the form
            ``projects/<project>/instances/<instance>``
        app_profile_id (str):
            Optional. This value specifies routing for preparing the
            query. Note that this ``app_profile_id`` is only used for
            preparing the query. The actual query execution will use the
            app profile specified in the ``ExecuteQueryRequest``. If not
            specified, the ``default`` application profile will be used.
        query (str):
            Required. The query string.
        proto_format (google.cloud.bigtable_v2.types.ProtoFormat):
            Protocol buffer format as described by
            ProtoSchema and ProtoRows messages.

            This field is a member of `oneof`_ ``data_format``.
        param_types (MutableMapping[str, google.cloud.bigtable_v2.types.Type]):
            Required. ``param_types`` is a map of parameter identifier
            strings to their ``Type``\ s.

            In query string, a parameter placeholder consists of the
            ``@`` character followed by the parameter name (for example,
            ``@firstName``) in the query string.

            For example, if param_types["firstName"] = Bytes then
            @firstName will be a query parameter of type Bytes. The
            specific ``Value`` to be used for the query execution must
            be sent in ``ExecuteQueryRequest`` in the ``params`` map.
    """

    instance_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    app_profile_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    query: str = proto.Field(
        proto.STRING,
        number=3,
    )
    proto_format: data.ProtoFormat = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="data_format",
        message=data.ProtoFormat,
    )
    param_types: MutableMapping[str, types.Type] = proto.MapField(
        proto.STRING,
        proto.MESSAGE,
        number=6,
        message=types.Type,
    )


class PrepareQueryResponse(proto.Message):
    r"""Response message for Bigtable.PrepareQueryResponse

    Attributes:
        metadata (google.cloud.bigtable_v2.types.ResultSetMetadata):
            Structure of rows in the response stream of
            ``ExecuteQueryResponse`` for the returned
            ``prepared_query``.
        prepared_query (bytes):
            A serialized prepared query. Clients should treat this as an
            opaque blob of bytes to send in ``ExecuteQueryRequest``.
        valid_until (google.protobuf.timestamp_pb2.Timestamp):
            The time at which the prepared query token
            becomes invalid. A token may become invalid
            early due to changes in the data being read, but
            it provides a guideline to refresh query plans
            asynchronously.
    """

    metadata: data.ResultSetMetadata = proto.Field(
        proto.MESSAGE,
        number=1,
        message=data.ResultSetMetadata,
    )
    prepared_query: bytes = proto.Field(
        proto.BYTES,
        number=2,
    )
    valid_until: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )


__all__ = tuple(sorted(__protobuf__.manifest))
